/*
 * @(#) Jessic.java 98/11/23
 *
 * Copyright 1997, 1998 by SRG, Dept. of Computer Science, Hong Kong Univ.
 * Pokfulam RD, Hong Kong, China
 * All rights reserved.
 * 
 * This software is the condifential and prprietary information of SRG. 
 * You shall not disclose such confidential information and shall use it
 * only in accordance with the terms of the license agreement you got this
 * software.
 */

package CLUSTERPROBE.GUI;
 
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;
import java.net.URL;
import java.net.URLConnection;
import java.util.*;

import com.sun.java.swing.event.*;
import com.sun.java.swing.undo.*;
import com.sun.java.swing.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.border.*;

import CLUSTERPROBE.message.*;
import CLUSTERPROBE.resources.*;
/**
 * This main graphics interface of CLUSTERPROBE.
 * 
 * @author Jimmy Liang
 * @version 1.00 11/23/98
 */

public class Jessic extends JPanel {

	private JSplitPane comp;
	private JSplitPane subcomp;
	private JMenuBar menubar;
	private Hashtable menuItems;
	private JToolBar toolbar;
	private JScrollPane pview,sview;
	private JPanel eview;
	private JComponent status;

	private JTable eventTable;
	private JTable tableView;
	private TableSorter dataModel0;
	private TableSorter dataModel1;
	private TableSorter dataModel2;
	private TableSorter dataModel3;

	private int con = 2;
	private int showMode = 0;

	/**
	 * Suffix applied to the key used in property file 
	 * lookups for am inage.
	 */
	public static final String imageSuffix = "Image";

	/**
	 * Suffix applied to the key used in property file
	 * lookups for a label.
	 */
	public static final String labelSuffix = "Label";

	/**
	 * Suffix applied to the key used in property file
	 * lookups for tooltip text.
	 */
	public static final String tipSuffix="Tooltip"; 

	Jessic() {
		super(true);
		

		setBorder(BorderFactory.createEtchedBorder());
		setLayout(new BorderLayout());
		
		//Establish communications
		createCommunication();

		//Create the embedded components
		createComponent();
		comp.setFont(BundleGroup.defaultFont);

		menuItems = new Hashtable();
		menubar = createMenubar();

		menubar.add(Box.createHorizontalGlue());
		menubar.add(new JMenu("Help"));

		add("North",menubar);
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		//panel.add("North",createToolbar());
		panel.add("Center",comp);
		add("Center",panel);
	 	add("South",createStatusbar());

		setFont(BundleGroup.defaultFont);
	}	
	
	public static void main(String[] args) {

		//Force Jessic to come up in the Cross Platform L&F
		try {
			UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
			// If you want the system L&F instead, comment out the above line and uncomment the following:
			//UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}catch(Exception e){
			System.out.println("Error loading L&F: "+e);
		}

		try {
		String vers = System.getProperty("java.version");
		if (vers.compareTo("1.1.2") <0) {
			System.out.println("!!!WARNING: Swing must be run with a "+"1.1.2 or higher version VM!!!");
		}

		BundleGroup.setProperty();
		JFrame frame = new JFrame();
		frame.setTitle(BundleGroup.property.getString("Title"));
		frame.getContentPane().add("Center",new Jessic());
		WindowListener l = new WindowAdapter() {
			public void windowClosing(WindowEvent e) {System.exit(0);}
		};
		frame.addWindowListener(l);

		frame.pack();
		frame.setSize(500,600);
		frame.show();
		}catch(Throwable t) {
			System.out.println("uncaught exception: "+t);
			t.printStackTrace();
		}
	}

	/**
	 * Establish the communications with server, initial the EventTransceiver object.
	 *
	 */

	protected void createCommunication() {
		try {
System.out.println("Trying to connect server...");
			String server = BundleGroup.getResourceString("server");
			//String server = getClass().getResource("").getHost();
			int port = Integer.parseInt(BundleGroup.getResourceString("port"));

			System.out.println("Connect to resources management server "+ server +" on port "+Integer.toString(port));
			new EventTransceiver(server,port);
		}catch(Exception e) {
			System.out.println("Connect server fail:"+e.toString());
			System.exit(-1);
		}
	}
	

	/**
	 * Create a splited pane to represent the physics location, nodes status and coming events.
	 *
	 */
	protected void createComponent() {
		pview = createLogicView();
		sview = createEventTable();
		eview = createCon();

		Dimension minimumSize = new Dimension(10,30);
		pview.setMinimumSize(minimumSize);
		sview.setMinimumSize(minimumSize);
		eview.setMinimumSize(minimumSize);

		pview.setPreferredSize(new Dimension(800,300));

		subcomp = new JSplitPane(JSplitPane.VERTICAL_SPLIT,pview,sview);

		comp = new JSplitPane(JSplitPane.VERTICAL_SPLIT,subcomp,eview);
		comp.setContinuousLayout(true);
		comp.getAccessibleContext().setAccessibleName("Split the subview and event view");		
		comp.setOneTouchExpandable(true);
		comp.setDividerLocation(0.8);
		comp.setDividerSize(10);
		comp.setPreferredSize(new Dimension(800,500));

		subcomp.setContinuousLayout(true);
		subcomp.getAccessibleContext().setAccessibleName("Split the statues view and event view");
		subcomp.setOneTouchExpandable(true);
		subcomp.setDividerLocation(0.75);
		subcomp.setDividerSize(10);
		subcomp.setPreferredSize(new Dimension(800,400));

		pview.setBackground(Color.white);
		eview.setBackground(Color.yellow);
		sview.setBackground(Color.yellow);
	}	
	

	/**
	 * Refresh logic view of the cluster.
	 *
	*/
	protected JScrollPane createLogicView() {
		return new JScrollPane(new LogicViewPane());
	}
			

	/**
	 * Create the menubar for the app. By default this pulls the definition of 
	 * the menu from the associated property file.
	 */	
	protected JMenuBar createMenubar()
	{
		JMenuItem mi;
		JMenuBar mb = new JMenuBar();

		String[] menuKeys = tokenize(BundleGroup.getResourceString("menubar"));
		for (int i=0;i<menuKeys.length;i++)
		{
			JMenu m = createMenu(menuKeys[i]);
			if (m!=null)
				mb.add(m);
		}
		return mb;
	}

	/**
	 * Take the given string and chop it up into a series
	 * of strings on whitespace boundries. This is usefule
	 * for trying to get an array of strings out of the 
	 * property file.
         */
	protected String[] tokenize(String input) {
		Vector v = new Vector();
		StringTokenizer t = new StringTokenizer(input);
		String cmd[];

		while(t.hasMoreTokens())
			v.addElement(t.nextToken());
		cmd = new String[v.size()];
		for (int i=0;i<cmd.length;i++)
			cmd[i] = (String)v.elementAt(i);
		return cmd;
	}
	

	/**
	 * Create a menu for the apps. By default this pulls the
	 * definiton of the menu from the associated property file. 
	 */
	protected JMenu createMenu(String key) {
		String[] itemKeys = tokenize(BundleGroup.getResourceString(key));
		JMenu menu = new JMenu(BundleGroup.getResourceString(key+"Label"));
		for (int i=0;i<itemKeys.length;i++) {
			if (itemKeys[i].equals("-")) {
				menu.addSeparator();
			} else {
				JMenuItem mi = createMenuItem(itemKeys[i]);
				menu.add(mi);
			}	
		}
		return menu;
	}

	final String[] names = {"JobID", "User","Stat","Type","Req_Node_No.","Exec_hosts","Sub_Time","Job Description"};

	//long values
	Object[] longValues={new Integer(12345),"XXXXXXXXXX","PEND","DEDICATED","111","XXXXXXXXXXXX","XXXXXXXXXXXX","XXXXXXXXXX"};

	// Create the dummy data ( just for show)
	Object[][] data = {
		{new Integer(1230), "David", "PEND", "normal",new Integer(4),"","Dec 2 16:34", "RayTrace"},
		{new Integer(1214), "atctam", "PEND", "night", new Integer(1),"","Dec 2 15:58", "BenchMark"},
		{new Integer(1123), "cmlee", "PEND", "dedicated", new Integer(3),"","Dec 2 15:56", "job11"},
		{new Integer(1122), "jimmy", "PEND", "night", new Integer(1),"","Dec 2 15:46", "job10"},
		{new Integer(1120), "kpwong", "PEND", "night", new Integer(2),"","Dec 2 15:29", "dptest"},
		{new Integer(1108), "jimmy", "PSUS", "dedicated", new Integer(1),"","Dec 2 15:19", "job09"},
		{new Integer(1102), "ydsun", "RUN", "high-priority", new Integer(5),"c1,c3,c5,c6,c7","Dec 2 15:09", "RayTrace"},
		{new Integer(1209), "ydsun", "RUN", "dedicated", new Integer(2),"c2,c4","Dec 2 15:05", "job3"},
		{new Integer(1132), "jimmy", "RUN", "normal", new Integer(1),"c12","Dec 2 14:25", "myjob3"},
		{new Integer(1122), "ydsun", "RUN", "normal", new Integer(1),"c8","Dec 2 13:54", "job15"},
		{new Integer(1123), "cmlee", "RUN", "normal", new Integer(1),"c10","Dec 2 12:26", "compile"},
		{new Integer(1243), "cmlee", "RUN", "normal", new Integer(1),"c11","Dec 2 15:56", "build"},
		{new Integer(1003), "atctam","RUN", "dedicated", new Integer(1),"c16","Dec 2 15:56", "job1"}
		};

	/**
	 * This is the method to create the event status table
	 */

	public JScrollPane createEventTable() {

		// Create a model of the data
		TableModel dataModel = new AbstractTableModel() {
			public int getColumnCount() {return names.length;}
			public int getRowCount() {return data.length;}
			public Object getValueAt(int row, int col) {return data[row][col];}
			public String getColumnName(int column) { return names[column];}
			public Class getColumnClass(int c) { return getValueAt(0,c).getClass();}
			public boolean isCellEditable(int row, int col) { return false;
			}
			public void setValueAt(Object aValue, int row, int column) { data[row][column] = aValue;}
		};

		// Create the table
		TableSorter sorter = new TableSorter(dataModel);
		eventTable = new JTable(sorter);
		sorter.addMouseListenerToHeaderInTable(eventTable);
		
		eventTable.setRowHeight(15);
		eventTable.setBackground(Color.yellow);
		eventTable.setAutoResizeMode(0);
		eventTable.setShowGrid(false);
		eventTable.sizeColumnsToFit(true);
		eventTable.setFont(BundleGroup.boldFont);

		resizeTable(eventTable,longValues);

		return new JScrollPane(eventTable);
	}
		
	public void resizeTable(JTable table,Object[] lValues)
	{
		TableModel dataModel = table.getModel();
		TableColumn column = null;
		Component cp = null;
		int headerWidth = 0;
		int cellWidth=0;
		int colCount = dataModel.getColumnCount();

		for (int i=0; i<colCount;i++)
		{
			column = table.getColumnModel().getColumn(i);
			cp = column.getHeaderRenderer().getTableCellRendererComponent(null,column.getHeaderValue(),false,false,0,0);
			headerWidth = cp.getPreferredSize().width;
			cp = table.getDefaultRenderer(dataModel.getColumnClass(i)).getTableCellRendererComponent(table,lValues[i],false,false,0,i);		
			cellWidth = cp.getPreferredSize().width;
			column.setMinWidth(Math.max(headerWidth,cellWidth));
		}
	}
			
		
	/**
	 * Host table includes the configuration, load information and 
	 * incoming events details about each node in cluster. 
	 */

	public JPanel createCon() 
	{
		JPanel mainPanel = new JPanel(new BorderLayout());

		JPanel controlPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));

		JComboBox box1 = new JComboBox();
		JComboBox box2 = new JComboBox();
		box1.setFont(BundleGroup.defaultFont);
		box2.setFont(BundleGroup.defaultFont);

		box1.addItem("Node Configuration");
		box1.addItem("Pool Configuration");
		box1.addItem("Host Load"); 
		box1.addItem("Events"); 
		box1.setSelectedIndex(con);

		box1.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				JComboBox source = (JComboBox)e.getSource();
				con = source.getSelectedIndex();
				if (con==0)
				{
					tableView.setModel(dataModel0);
					resizeTable(tableView,longValues0);
				}
				if (con==1)
				{
					tableView.setModel(dataModel1);
					resizeTable(tableView,longValues1);
				}
				if (con==2)
				{
					tableView.setModel(dataModel2);
					resizeTable(tableView,longValues2);
				}
				if (con==3)
				{
					tableView.setModel(dataModel3);
					resizeTable(tableView,longValues3);
				}
				tableView.repaint();
			}
		});

		box2.addItem("Show by table");
		box2.addItem("show by chart");

/*
		JPanel title1 = new JPanel();
		JPanel title2 = new JPanel();
		title1.setBorder(new TitledBorder("Selection"));
		title2.setBorder(new TitledBorder("Show Mode"));

		title1.add(box1);
		title2.add(box2);

		controlPanel.add(title1);
		controlPanel.add(title2);
*/

		controlPanel.add(box1);
		controlPanel.add(box2);

		mainPanel.add(controlPanel,BorderLayout.NORTH);

		mainPanel.add(createConTableOrChart(),BorderLayout.CENTER);

		//return new JScrollPane(mainPanel);
		return mainPanel;
	}

	//final
	final String[] names0 = {"Host Name","Ip Address","Machine Type","CPU","OS","Harddisk Size","Memory Size","Network Cards"};
	final String[] names1 = {"Pool Name","Pool Type","include_nodes","OverLap","stat","Description"};
	final String[] names2 = {"Host Name","15-sec","1-min","15-min","CPU","Paging","disk I/O","Login No.","/tmp free","swap free","memory free"};
	final String[] names3 = {"Event ID","Date","Event Type","Description","From","Emergency Degree","Auto handled"};

	Object[][] data0 = {
		{"c1","202.45.128.111","Wchip","1xIntel Pentium","LINUX 2.0.32",new Integer(500),new Integer(32),"tulip,3c5x9"},
		{"c2","202.45.128.112","Wchip","1xIntel Pentium","LINUX 2.0.32",new Integer(500),new Integer(32),"tulip","3c5x9"},
		{"c3","202.45.128.80","alr","4xIntel PentiumPro","LINUX 2.0.32",new Integer(5000),new Integer(264),"tulip,eepro100"},
		{"c4","202.45.128.113","Wchip","1xIntel Pentium","LINUX 2.0.32",new Integer(500),new Integer(32),"tulip"},
		{"c5","202.45.128.114","Wchip","1xIntel Pentium","LINUX 2.0.32",new Integer(500),new Integer(32),"tulip"},
		{"c6","202.45.128.115","Wchip","1xIntel Pentium","LINUX 2.0.32",new Integer(500),new Integer(32),"tulip"},
		{"c7","202.45.128.116","Wchip","1xIntel Pentium","LINUX 2.0.32",new Integer(500),new Integer(32),"tulip"}
	};

	Object[] longValues0 = {"XXXXXXXX","255.255.255.255","XXXXXXXXXXXX","XXXXXXXXXXXXXXXX","XXXXXXXXXXXXXX","XXXX","XXXX","XXXXXXXXXXXXXXXXXX"};

	Object[][] data1 = {
		{"batch1","batch","c1-c8","5","enabled","for sequetial and parallel batch jobs"},
		{"batch2","batch&parallel","c9-c12","1","enabled","for parallel batch jobs"},
		{"Login","Login&interactive","c13-c16","","enabled","User can login these machines"}
	};
	Object[] longValues1 = {"XXXXXXXXXX","XXXXXXXXXXXXXXXXX","XXXXXXXXXXXXXXXX","XX","XXXXXXXX","XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"};

	Object[][] data2 = {
		{"",new Float(-1.0),new Float(-1.0),new Float(-1.0),"",new Float(-1.0),new Integer(-1),new Integer(-1),new Integer(-1),new Integer(-1),new Integer(-1)}
	};
/*
	Object[][] data2 = {
		{"c1",new Float(0.0),new Float(0.25),new Float(1.1),"45%",new Float(0.09),new Integer(51),new Integer(2),new Integer(17),new Integer(545),new Integer(394)},
		{"c2",new Float(0.3),new Float(0.2),new Float(1.1),"25%",new Float(0.12),new Integer(53),new Integer(0),new Integer(13),new Integer(321),new Integer(432)},
		{"c3",new Float(0.3),new Float(0.25),new Float(1.0),"65%",new Float(0.12),new Integer(45),new Integer(3),new Integer(32),new Integer(143),new Integer(421)},
		{"c4",new Float(0.0),new Float(0.20),new Float(0.6),"35%",new Float(0.08),new Integer(49),new Integer(3),new Integer(13),new Integer(251),new Integer(234)},
		{"c5",new Float(1.0),new Float(1.2),new Float(1.3),"42%",new Float(0.09),new Integer(54),new Integer(1),new Integer(14),new Integer(453),new Integer(246)},
		{"c6",new Float(0.22),new Float(0.34),new Float(0.2),"61%",new Float(0.11),new Integer(55),new Integer(0),new Integer(7),new Integer(511),new Integer(342)},
		{"c7",new Float(0.14),new Float(1.23),new Float(1.3),"87%",new Float(0.08),new Integer(45),new Integer(0),new Integer(22),new Integer(521),new Integer(321)},
		{"c8",new Float(1.0),new Float(0.31),new Float(1.1),"45%",new Float(0.09),new Integer(51),new Integer(3),new Integer(17),new Integer(545),new Integer(394)}
	};
*/

	Object[] longValues2={"XXXXXX","XXXX","XXXX","XXXX","XXXX","XXXX","XXXX","XXX","XXXX","XXXX","XXXX"};

	Object[][] data3 = {
		{new Integer(111),"Dec 11, 11:22", "CPU.OverLoad","CPU Util. >0.95","c3","2","no"},
		{new Integer(110),"Dec 11, 11:12", "SWAP.FULL","Swap used Util>0.95","c2","3","no"},
		{new Integer(109),"Dec 11, 11:02", "Process.DP.untouch","DP erroe","c12","1","yes"},
		{new Integer(108),"Dec 11, 10:22", "Process.DP.untouch","DP erroe","c11","1","yes"},
		{new Integer(105),"Dec 11, 10:02", "Network.ip.errorPackage","errror ip package in 1h >2000","c4","3","no"},
		{new Integer(104),"Dec 11, 0:33", "Network.unvail","node untouchable","c6","1","no"}
	};
	Object[] longValues3 = {"XXXX","XXXXXXXXXXXXXXXX","XXXXXXXXXXXX","XXXXXXXXXXXXXXXXXXXXX","XXXXXX","XX","XXX"};

		
	public JScrollPane createConTableOrChart() {
		TableModel model0 = new AbstractTableModel() {
			public int getColumnCount() {
				return names0.length;
			}
			public int getRowCount() {
				return data0.length;
			}
			public Object getValueAt(int row, int col) {
				return data0[row][col];
			}
			public String getColumnName(int column) {
				return names0[column];
			}
			public Class getColumnClass(int c) {
				return getValueAt(0,c).getClass();
			}
			public boolean isCellEditable(int row, int col) {
				return false;
			}
			public void setValueAt(Object aValue, int row, int column)
			{
				data0[row][column]=aValue;	
			}
		};
		dataModel0 = new TableSorter(model0);

		TableModel model1 = new AbstractTableModel() {
			public int getColumnCount() {
				return names1.length;
			}
			public int getRowCount() {
				return data1.length;
			}
			public Object getValueAt(int row, int col) {
				return data1[row][col];
			}
			public String getColumnName(int column) {
				return names1[column];
			}
			public Class getColumnClass(int c) {
				return getValueAt(0,c).getClass();
			}
			public boolean isCellEditable(int row, int col) {
				return false;
			}
			public void setValueAt(Object aValue, int row, int column)
			{
				data1[row][column]=aValue;	
			}
		};
		dataModel1 = new TableSorter(model1);

		TableModel model2 = new AbstractTableModel() {
			public int getColumnCount() {
				return names2.length;
			}
			public int getRowCount() {
				return data2.length;
			}
			public Object getValueAt(int row, int col) {
				return data2[row][col];
			}
			public String getColumnName(int column) {
				return names2[column];
			}
			public Class getColumnClass(int c) {
				return getValueAt(0,c).getClass();
			}
			public boolean isCellEditable(int row, int col) {
				return false;
			}
			public void setValueAt(Object aValue, int row, int column)
			{
				data2[row][column]=aValue;	
			}
		};
		dataModel2 = new TableSorter(model2);

		TableModel model3 = new AbstractTableModel() {
			public int getColumnCount() {
				return names3.length;
			}
			public int getRowCount() {
				return data3.length;
			}
			public Object getValueAt(int row, int col) {
				return data3[row][col];
			}
			public String getColumnName(int column) {
				return names3[column];
			}
			public Class getColumnClass(int c) {
				return getValueAt(0,c).getClass();
			}
			public boolean isCellEditable(int row, int col) {
				return false;
			}
			public void setValueAt(Object aValue, int row, int column)
			{
				data3[row][column]=aValue;	
			}
		};
		dataModel3 = new TableSorter(model3);

		tableView = null;
		if (con==0)
		{
			tableView = new JTable(dataModel0);
			resizeTable(tableView,longValues0);
		}
		else if (con==1)
		{
			tableView = new JTable(dataModel1);
			resizeTable(tableView,longValues1);
		}
		else if (con==2)
		{
			tableView = new JTable(dataModel2);
			resizeTable(tableView,longValues2);
		}
		else 
		{
			tableView = new JTable(dataModel3);
			resizeTable(tableView,longValues3);
		}

		dataModel0.addMouseListenerToHeaderInTable(tableView);
		dataModel1.addMouseListenerToHeaderInTable(tableView);
		dataModel2.addMouseListenerToHeaderInTable(tableView);
		dataModel3.addMouseListenerToHeaderInTable(tableView);

		tableView.setRowHeight(15);
		tableView.setBackground(Color.white);
		tableView.setAutoResizeMode(0);
		tableView.setShowGrid(true);
		tableView.sizeColumnsToFit(true);
		tableView.setFont(BundleGroup.boldFont);
		
		// handle data transfered from server
		if (ResourceRefreshEventHandler.current == null)
		{
			new ResourceRefreshEventHandler();
			EventTransceiver.current.addHandler(ResourceRefreshEventHandler.current,new ResourceRefreshEvent());
			EventTransceiver.current.addHandler(ResourceRefreshEventHandler.current,new ErrorEvent());
		}
		ResourceRefreshOperator op = new ResourceRefreshOperator() {
		public String[] getInterestResources()
		{
			String[] interest = {"CLUSTERPROBE.resources.Cluster_NodeLists","CLUSTERPROBE.resources.Cluster_Workstation_StatusTable"};
			return interest;
		}
		
		public void operateError(ErrorEvent e) {
			return;
		}

		public void operate(Vector dv,Object location)
		{
			Enumeration de = dv.elements();
			try {
			boolean cleared = false;	
			int nn = 0;
			while (de.hasMoreElements())
			{
				ResourceData rd = (ResourceData) de.nextElement();
				String rn = rd.getName();		
				
				//for Node Lists
				if (rn.equals("CLUSTERPROBE.resources.Cluster_NodeLists"))
				{
					if (!cleared)
					{
						cleared = true;
						hosts.removeAllElements();
						alias.removeAllElements();
						inPools.removeAllElements();
						nn = 1;
					}	
					ResourceTableData data = (ResourceTableData) rd;
					Vector vt = data.getRecord();
					hosts.addElement(nl.getValue(0,vt));
					alias.addElement(nl.getValue(1,vt));
					inPools.addElement(nl.getValue(2,vt));
				}

				// for Status Table
				if (rn.equals("CLUSTERPROBE.resources.Cluster_Workstation_StatusTable"))
				{	
					nn=3;
					break;
				}
			}

			if (nn==1)
			{
				if (data0==null) {
					data0 = new Object[hosts.size()][];
					for (int i=0;i<data0.length;i++)
					{
						data0[i] = new Object[names0.length];
						data0[i][0] = (String) hosts.elementAt(i);
						data0[i][1] = "";	
						data0[i][2] = "";	
						data0[i][3] = "";	
						data0[i][4] = "";	
						data0[i][5] = new Integer(0);;
						data0[i][6] = new Integer(0);
						data0[i][7] = "";
					}
				}
				else
				{
					Object[][] dd = new Object[hosts.size()][];
					int count = 0;
					for (int i=0;i<data0.length;i++)
						if (hosts.contains(data0[i][0]))
							dd[count++] = data[i];
					Enumeration he = hosts.elements();
					while (he.hasMoreElements())
					{
						String hostName = (String) he.nextElement();
						int i=0;
						for (;i<data0.length;i++)
							if (hostName.equals(data0[i][0]))
								break;
						if (i==data0.length)
						{
							dd[count] = new Object[names0.length];
							dd[count][0] = hostName;
							dd[count][1] = "";	
							dd[count][2] = "";	
							dd[count][3] = "";	
							dd[count][4] = "";	
							dd[count][5] = new Integer(0);
							dd[count][6] = new Integer(0);
							dd[count++][7] = "";
						}
					}
					data0 = dd;
					dataModel0.tableChanged(new TableModelEvent(dataModel0));
				}
			}
			else if (nn==3)
			{
				if (data2==null)
				{
					data2 = new Object[dv.size()][];
					for (int i=0;i<data2.length;i++)
					{
						ResourceTableData data = (ResourceTableData)dv.elementAt(i);
						Vector vt = data.getRecord();
						String hostName = (String) statusTable.getValue(0,vt);
						for (int j=0;j<hosts.size();j++)	
							if (hostName.equals((String)alias.elementAt(j)))
								hostName = (String) hosts.elementAt(j);
						data2[i] = new Object[names2.length];
						data2[i][0] = hostName;
						Integer state = (Integer) statusTable.getValue(1,vt);
						if (state.intValue()<statusTable.ERROR)
						{
							data2[i][1] = statusTable.getValue(3,vt);
							data2[i][2] = statusTable.getValue(4,vt);
							data2[i][3] = statusTable.getValue(5,vt);
						}
						else
						{
							data2[i][1] = new Float(-1.0);
							data2[i][2] = new Float(-1.0);
							data2[i][3] = new Float(-1.0);
						}
						data2[i][4] = "";
						data2[i][5] = new Float(-1.0);
						data2[i][6] = new Integer(-1);
						data2[i][7] = new Integer(-1);
						data2[i][8] = new Integer(-1);
						data2[i][9] = new Integer(-1); 
						data2[i][10] = new Integer(-1);
					}	
				}
				else {
					Object[][] dd = new Object[dv.size()][];
					for (int i=0;i<dd.length;i++)
					{
						ResourceTableData data = (ResourceTableData)dv.elementAt(i);
						Vector vt = data.getRecord();
						String hostName = (String) statusTable.getValue(0,vt);
						for (int j=0;j<hosts.size();j++)	
							if (hostName.equals((String)alias.elementAt(j)))
								hostName = (String) hosts.elementAt(j);
						int j=0;
						for (;j<data2.length;j++)
							if (hostName.equals(data2[j][0]))
							{
								dd[i] = data2[j];	
								break;
							}
						if (j==data2.length)
						{
							dd[i] = new Object[names2.length];
							dd[i][0] = hostName;
							dd[i][4] = "";
							dd[i][5] = new Float(-1.0);
							dd[i][6] = new Integer(-1);
							dd[i][7] = new Integer(-1);
							dd[i][8] = new Integer(-1);
							dd[i][9] = new Integer(-1);
							dd[i][10] = new Integer(-1);
						}
						Integer state = (Integer) statusTable.getValue(1,vt);
						if (state.intValue()<statusTable.ERROR)
						{
							dd[i][1] = statusTable.getValue(3,vt);
							dd[i][2] = statusTable.getValue(4,vt);
							dd[i][3] = statusTable.getValue(5,vt);
						}
						else {
							dd[i][1] = new Float(-1.0);
							dd[i][2] = new Float(-1.0);
							dd[i][3] = new Float(-1.0);
						}
					}
					data2 = dd;
					dataModel2.tableChanged(new TableModelEvent(dataModel2));
				}
			}
						
			}catch(Exception e) {System.out.println(e);};
		}
		};
		ResourceRefreshEventHandler.current.register(op);	
		return new JScrollPane(tableView);
	}

	Cluster_NodeLists nl = new Cluster_NodeLists();
	Cluster_Workstation_StatusTable statusTable = new Cluster_Workstation_StatusTable();
	Vector hosts = new Vector();
	Vector alias = new Vector();
	Vector inPools = new Vector();
	/**
	 * This is the hook throught which all menu item are created.
	 * It registers the result with the menuitem hashtable so that
	 * it can be fetched whith getMenuItem().
	 * @see #getmenuItem
	 */
	protected JMenuItem createMenuItem(String cmd) {
		JMenuItem mi = new JMenuItem(BundleGroup.getResourceString(cmd+labelSuffix));
		URL url = BundleGroup.getResource(cmd+imageSuffix);
		if (url!=null) {
			mi.setHorizontalTextPosition(JButton.RIGHT);
			mi.setIcon(new ImageIcon(url));	
		}
/*
		String astr = BundleGroup.getResourceString(cmd+actionSuffix);
		if (astr = null) {
			astr = cmd;
		}
		mi.setActionCommand(astr);
		Action a = getAction(astr);
		if (a!=null) {
			mi.addActionListener(a);
			a.addPropertyChangeListener(createActionChangeListener(mi));
			mi.setEnabled(a.isEnabled());
		} else {
			mi.setEnabled(false);
		}
*/
		
		menuItems.put(cmd,mi);
		return mi;
	}
		
	
	/** 
	 * Create the toolbar. By default this reads the
	 * property file for the definition of the toolbar.
	 */
	private JComponent createToolbar() {
		toolbar = new JToolBar();
		String[] toolKeys = tokenize(BundleGroup.getResourceString("toolbar"));
		for (int i=0;i<toolKeys.length;i++)
		{
			if (toolKeys[i].equals("-")) {
				toolbar.add(Box.createHorizontalStrut(5));
			} else
			{
				toolbar.add(createTool(toolKeys[i]));
			}
		}
		toolbar.add(Box.createHorizontalGlue());
		return toolbar;
	}

	/**
	 * Hook through which every toolbar item is created.
	 */
	protected JComponent createTool(String key) {
		return createToolbarButton(key);
	}

	/**
	 * create a button to go inside of the toolbar. By default this
	 * will load an image resource. The image filename is relative to
	 * the classpath(include the '.' directory if its a part of the
	 * classpath), and may either be in a JAR file or a separate file.
	 *
	 * @param key The key in the property file to serve as the basis 
	 * of lookups.
	 */
	protected JButton createToolbarButton(String key) {
		URL url = BundleGroup.getResource(key+imageSuffix);
		JButton b = new JButton(new ImageIcon(url)) {
			public float getAlignmentY() { return 0.5f; }
		};
		b.setRequestFocusEnabled(false);
		b.setMargin(new Insets(1,1,1,1));

/*
		String astr = BundleGroup.getResourceString(key+actionSuffix);
		if (astr = null) {
			astr = key;
		}
		Action a = getAction(astr);
		if (a!=null) {
			b.setActionCommand(astr);
			b.addActionListener(a);	
		} else
		{
			b.setEnabled(false);
		}
*/

		String tip = BundleGroup.getResourceString(key+tipSuffix);
		if (tip !=null) {
			b.setToolTipText(tip);
		}

		return b;
	}
		

	/**
	 * Create a status bar
	 */
	protected JComponent createStatusbar()
	{
		//need to do something reasonable here
		status = new StatusBar();
		return status;
	}
			
	/**
	 * the Status Bar applied in this app
	 */
	class StatusBar extends JComponent {
		public StatusBar() {
			super();
			setLayout(new BoxLayout(this,BoxLayout.X_AXIS));
		}

		public void paint(Graphics g) {
			super.paint(g);
		}
	}
	
}	
