/*
 * @(#) LogicViewPane.java 98/11/23
 *
 * Copyright 1997, 1998 by SRG, Dept. of Computer Science, Hong Kong Univ.
 * Pokfulam RD, Hong Kong, China
 * All rights reserved.
 * 
 * This software is the condifential and prprietary information of SRG. 
 * You shall not disclose such confidential information and shall use it
 * only in accordance with the terms of the license agreement you got this
 * software.
 */

package CLUSTERPROBE.GUI;
 
import java.net.URL;
import java.awt.event.*;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.GridLayout;
import java.util.*;
import java.awt.Color;

import com.sun.java.swing.ImageIcon;
import com.sun.java.swing.JPanel;
import com.sun.java.swing.JLabel;
import com.sun.java.swing.border.*;
import com.sun.java.swing.JComponent;
import com.sun.java.swing.JOptionPane;

import CLUSTERPROBE.message.*;
import CLUSTERPROBE.resources.Cluster_Workstation_StatusTable;

/**
 * Show the logic view of current cluster with the statues of each workstation.
 * 
 * @author Jimmy Liang
 * @version 1.00 12/3/98
 */

public class LogicViewPane extends JPanel implements ResourceRefreshOperator
{

	public static ImageIcon READY_ICON;
	public static ImageIcon ERROR_ICON;
	public static ImageIcon WARNING_ICON;
	public static String[] interestResources = {"CLUSTERPROBE.resources.Cluster_Workstation_StatusTable"};
	Cluster_Workstation_StatusTable resource = new Cluster_Workstation_StatusTable();

	Hashtable status = new Hashtable();
	
	public LogicViewPane() {
		super();
		getWorkstationIcon();
		if (ResourceRefreshEventHandler.current == null ) 
		{
			new ResourceRefreshEventHandler();
			EventTransceiver.current.addHandler(ResourceRefreshEventHandler.current,new ResourceRefreshEvent());
			EventTransceiver.current.addHandler(ResourceRefreshEventHandler.current, new ErrorEvent());
		}
		ResourceRefreshEventHandler.current.register(this);
	};

	/**
	 * Get the workstation icons from associate file
	*/
	protected void getWorkstationIcon() {
		try {
		URL url = BundleGroup.getResource("error");
		if (url!=null)
			setIcon(Cluster_Workstation_StatusTable.ERROR, new ImageIcon(url));
		url = BundleGroup.getResource("warning");
		if (url!=null)
			setIcon(Cluster_Workstation_StatusTable.WARNING, new ImageIcon(url));
		url = BundleGroup.getResource("ready");
		if (url!=null)
			setIcon(Cluster_Workstation_StatusTable.READY, new ImageIcon(url));
		}catch(Exception mfe) {
			System.out.println(mfe.toString());
		}
	}
			
	/**
	 * Return the resources which this operator interested.
	 *
	 */
	public String[] getInterestResources() {
		return interestResources;
	}

	/**
	 * Do operation when refreshing data coming
	 *
	 */
	public void operate(Vector dv, Object Location) {
		Enumeration de = dv.elements();
		String name;
		int state;
		String des;

		clear();
		while (de.hasMoreElements())
		{
			try {
			ResourceTableData  data = (ResourceTableData) de.nextElement();
			Vector vt = data.getRecord();

			name = (String)data.getKey();
			state = ((Integer)resource.getValue(resource.getId("state"),vt)).shortValue();
			des = (String)resource.getValue(resource.getId("description"),vt);
			addWorkstation(name,state,des);
			}catch(Exception e) {
				System.out.println(e.toString());
			};
		}
		arrangeLayout();
	}
				
	/**
	 * Do operation when error message coming
	 *
	 */
	public void operateError(ErrorEvent e)
	{
		if (e==null) return;
		JOptionPane.showMessageDialog(null,e.getErrMessage(),e.getName()+" alert",JOptionPane.ERROR_MESSAGE);
	}
			
	public int addWorkstation(String name, int state, String des)
	{
		return addWorkstation(name,state,des,false);
	}

	public int addWorkstation(String name,int state,String des,boolean force)
	{
		if (status.containsKey(name))  
			if (force)
				status.remove(name);
			else
				return ErrorMessage.NODE_EXISTED;
		Vector vt = new Vector(3);
		vt.addElement(new Integer(state));
		vt.addElement(des);
		vt.addElement(createIconLabel(name,state,des));
		status.put(name,vt);
		return 0;
	}

	public void clear()
	{
		status.clear();
	}

	public void removeWorkstation(String name)
	{
		status.remove(name);
	}

	public int modifyWorkstation(String name, int state, String des)
	{
		if (!status.containsKey(name))
			return ErrorMessage.NODE_NOT_EXISTED;
		Vector vt = (Vector) status.get(name);
		if (state!=0) 
			vt.setElementAt(new Integer(state),0);
		if (des!=null)
			vt.setElementAt(des,1);
		JLabel label = (JLabel) vt.elementAt(2);		
		label.setIcon(toIcon(name));
		label.setText(name+"("+getDesc(name)+")");
		return 0;
	}
			
	public void setIcon(int type, ImageIcon icon) {
		switch(type)
		{
		case Cluster_Workstation_StatusTable.READY:
			READY_ICON = icon;
			break;
		case Cluster_Workstation_StatusTable.WARNING:
			WARNING_ICON = icon;
			break;
		case Cluster_Workstation_StatusTable.ERROR:
			ERROR_ICON = icon;
			break;
		}
	}	

	public int getWorkstationCount() {
		if (status==null) return 0;
		return status.size();
	}

	public String[] getWorkstationNames()
	{
		int count = getWorkstationCount();
		if (count==0) return null;
		String[] names = new String[getWorkstationCount()];
		Enumeration keys = status.keys();
		int i= 0;	
		while (keys.hasMoreElements())
			names[i++] = (String) keys.nextElement();
		return names;
	}	
		 
	public int getType(String name) {
		Vector vt = (Vector) status.get(name);
		if (vt==null) return 0;
		return ((Integer)vt.elementAt(0)).shortValue();
	}

	public String getDesc(String name) {
		Vector vt = (Vector) status.get(name);
		if (vt==null) return null;
		return (String)vt.elementAt(1);
	}

	public ImageIcon toIcon(String name) {
		return getIcon(getType(name));
	}

	public ImageIcon getIcon(int type) {
		if ( type > Cluster_Workstation_StatusTable.ERROR)
			type = Cluster_Workstation_StatusTable.ERROR;
		else if (type>Cluster_Workstation_StatusTable.WARNING)
			type = Cluster_Workstation_StatusTable.WARNING;

		switch(type)
		{
		case Cluster_Workstation_StatusTable.READY:
			return READY_ICON;
		case Cluster_Workstation_StatusTable.ERROR:
			return ERROR_ICON;
		case Cluster_Workstation_StatusTable.WARNING:
			return WARNING_ICON;
		default:
			return null;
		}
	}


	JLabel createIconLabel(String name, int type, String des)
	{
		JLabel iconLabel = new JLabel(getIcon(type));
		iconLabel.setFont(BundleGroup.defaultFont);
		iconLabel.setText(name+"("+des+")");
		iconLabel.setHorizontalTextPosition(JLabel.CENTER);
		iconLabel.setVerticalTextPosition(JLabel.BOTTOM);				iconLabel.setVerticalAlignment(JLabel.CENTER);
		iconLabel.setHorizontalAlignment(JLabel.CENTER);
		iconLabel.setPreferredSize(new Dimension(100,60));
		return iconLabel;
	}

	void arrangeLayout() {
		int row = 5;
		int col = 0;
		int count = status.size();

		removeAll();
		if ((count%row)==0)
			col = count/row;
		else
			col = count/row+1;
	
		setLayout(new GridLayout(row,col));
		setBackground(Color.white);
	
		Enumeration names = status.keys();
		while (names.hasMoreElements())
		{
			String name = (String) names.nextElement();
			Vector vt = (Vector) status.get(name);
			JLabel iconLabel = (JLabel) vt.elementAt(2);
			add(iconLabel);
		}
		invalidate();
		validate();
		repaint();
	}

}	

		
		
	
	

