/*
 * @(#) CServer.java 99/5/6
 */

package CLUSTERPROBE.Server;
 
import java.util.EventListener;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Calendar;
import java.net.Socket;
import java.io.*;
import CLUSTERPROBE.message.*;
import CLUSTERPROBE.resources.*;
import CLUSTERPROBE.monitor.*;

/**
 * This class acts as a monitor server for client written by C.  
 *
 * @author Jimmy Liang
 * @version 1.00 5/6/99
 */

public class CServerII extends ManageServer
{

public static Vector requestTable = new Vector();
public static EventHandler h;
static Hashtable clientTable = new Hashtable();
boolean DebugFlag = true;

	public CServerII(int port) {
		super(port);
		addLocalObservers();
	}
	
	public void addLocalObservers()
	{
		h = new EventHandler() 
		{
			public synchronized void handleEvent(JessicEvent evt) 
			{
				if (evt.getId()==JessicEvent.MONITORDATA)
				{
					if(DebugFlag) System.out.println("In CServerII:Get JessicEvent.MONITORDATA");
					CTransceiverII tran = null;
					try {
					MonitorDataEvent event = (MonitorDataEvent) evt;

					tran = getClient(event);
					if (tran!=null)
						tran.sendEvent(event);
					}catch(Exception e) {
						MonitorRegistryEvent reg = getClientRegistry((MonitorDataEvent)evt);
						if (reg!=null)
						{
							reg.setIndicator(MonitorRegistryEvent.UNREGISTER);
							MonitorUtil.unregister(reg);
						}
						if (tran!=null)
							removeClient(tran);
					};
				}
			}
		};
	}
			

	public void processConnection(Socket s) {
		try {
			System.out.println("CServerII:Establish a connection with a client");
			CTransceiverII tran = new CTransceiverII(s.getInputStream(),s.getOutputStream());
			requestTable.addElement(tran);
		}catch(Exception e) {
			System.out.println("Connect Error:"+e.toString());
		}
		System.out.println("Will be out of CServerII Thread.");
	};

	public static void addClient(CTransceiverII tran, MonitorRegistryEvent evt)
	{
		if (!clientTable.containsKey(tran))
			clientTable.put(tran,new Vector());

		Vector vt = (Vector) clientTable.get(tran);
		vt.addElement(evt);
	}

	public static Vector getClient(CTransceiverII tran)
	{
		return (Vector) clientTable.get(tran);
	}

	public static void removeClient(CTransceiverII tran, MonitorRegistryEvent evt)
	{
		if (clientTable.containsKey(tran))
		{
			Vector vt = (Vector) clientTable.get(tran);	
			vt.removeElement(evt);
		}
	}

	public static void removeClient(CTransceiverII tran)
	{
		clientTable.remove(tran);
	}


	public static MonitorRegistryEvent getClientRegistry(MonitorDataEvent evt)
	{
		Enumeration en = clientTable.keys();
		while (en.hasMoreElements())
		{
			Object obj = en.nextElement();
			Vector vt = (Vector)clientTable.get(obj);
			for (int i=0;i<vt.size();i++)
			{
				MonitorRegistryEvent e1 = (MonitorRegistryEvent) vt.elementAt(i);
				if (MonitorUtil.isEquals(e1,evt))
					return e1;
			}
		}
		return null;
	}

	public static CTransceiverII getClient(MonitorDataEvent evt)
	{
		Enumeration en = clientTable.keys();
		while (en.hasMoreElements())
		{
			Object obj = en.nextElement();
			Vector vt = (Vector)clientTable.get(obj);
			for (int i=0;i<vt.size();i++)
			{
				MonitorRegistryEvent e1 = (MonitorRegistryEvent) vt.elementAt(i);
				if (MonitorUtil.isEquals(e1,evt))
					return (CTransceiverII) obj;
			}
		}
		return null;
	}

}

class CTransceiverII extends Thread
{

// Our connection o a remove agent
InputStream evIn = null;
OutputStream evOut = null;
final static short BEGIN = 111;
final static short NEXT = 221;
final static short END = 222;
boolean DebugFlag = true;

	public CTransceiverII(InputStream in, OutputStream out) {
		setStreams(in,out);
		start();
	}

	void setStreams(InputStream in, OutputStream out)
	{
		evIn = in;
		evOut = out;
	}

	public OutputStream getOutputStream()
	{
		return evOut;
	}

	public InputStream getInputStream()
	{
		return evIn;
	}

	public void disconnect() {
		try {
		if (evIn!=null) evIn.close();
		if (evOut!=null) evOut.close();
		}catch(Exception e) {};
	}
	
	public void run() {
		
		if(DebugFlag) System.out.println("In CServerII Thread...");
		try {
			DataInputStream in = new DataInputStream(getInputStream());
			DataOutputStream out = new DataOutputStream(getOutputStream());

			while (true) {
					short type = in.readShort();
					short id = in.readShort();

					String rn = Resource.getNameFromID(id);
					if (rn!=null)
						handleEvent(type,rn);
			}
		}catch(Exception ex) {
			Vector vt = CServerII.getClient(this);
			if (vt!=null)
			{
				for (int ii=0;ii<vt.size();ii++)
				{
					MonitorRegistryEvent reg = (MonitorRegistryEvent) vt.elementAt(ii);
					if (reg!=null)
					{
						reg.setIndicator(MonitorRegistryEvent.UNREGISTER);
						MonitorUtil.unregister(reg);
					}
				}
				CServerII.removeClient(this);
			};
			ex.printStackTrace();
		};
 		if(DebugFlag) System.out.println("End of CServer Thread.");
	}

  
     void initOut(DataOutputStream out) throws Exception 
	{
		out.writeShort(BEGIN);	
	}	

	synchronized void handleEvent(short type, String rn)
	{
		try {

		switch (type)
		{
		case MonitorRegistryEvent.MONITOR_EVENT:
			if(DebugFlag) System.out.println("In CServerTransceiverII:Get JessicEvent.MONITOR_EVENT");
		case MonitorRegistryEvent.MONITOR_DATA:
			if(DebugFlag) System.out.println("...In CServerTransceiverII:Get JessicEvent.MONITOR_DATA");
		case MonitorRegistryEvent.NOREPLY:
			if(DebugFlag) System.out.println("...In CServerTransceiverII:Get JessicEvent.NOREPLY");
		
			MonitorRegistryEvent mon = new MonitorRegistryEvent(type);
			int[] entry = {1,2,3,4};
			String[] index = {"cpu"};
			String[] hosts = {"fast1"};
			mon.setLocation(rn,entry,index);
			mon.setCondition(new Condition());
			mon.setReaction(new React());
			mon.setTargets(hosts);
			if (MonitorUtil.register(mon,null))
			{
				Util.addLocalHandler(CServerII.h,rn);
				CServerII.addClient(this,mon);
			}
			break;
		}
		}catch(Exception ex) {
			System.out.println("Monitoring Registry Error:"+ex.toString());
		}
	} 

	public void sendEvent(MonitorDataEvent evt) throws Exception
	{
		
		MonitorRegistryEvent reg = CServerII.getClientRegistry((MonitorDataEvent)evt);
		if (reg==null) return;

		// here to startup resource object	
		Resource rs;
		String rn = evt.getResourceName();

		if (!Util.contains(rn))
		{
			rs = (Resource) Util.instanceClass(rn);
		}	
		else
		{
			rs = Util.getResource(rn);
		}

		send(rs,evt.getSourceName(),reg.getEntry(),(Vector)evt.getData());
	}	

	public synchronized void send(Resource rs, String s, int[] entry,Vector vt) throws Exception
	{
		System.out.println(s);
		System.out.println(entry);
		System.out.println(vt);
		System.out.println("------");
		int k = 1-1+1;
			DataOutputStream out = new DataOutputStream(getOutputStream());

			// write the begin indicator
			initOut(out);
if (k==1)
return;
			out.writeShort(BEGIN);
			out.flush();

			//write resource id
			short id = Resource.getIDFromName(rs.getResourceName());
			out.writeShort(id);
			out.flush();

			//write the location 
			writeString(out,s);		
			out.flush();

			//write length of vector
			out.writeShort(vt.size());
			out.flush();
			
			for (int i=0;i<vt.size();i++)
			{
				int location = vt.size()-1-i;
				ResourceTableData data = (ResourceTableData) vt.elementAt(location);
				writeRecord(rs,out,data);
			}

			out.flush();
	}

	void writeString(DataOutputStream out, String s) throws Exception
	{
		short count = 0;
		if (s==null)
		{
			out.writeShort(count);
			return;
		}
		
		count = (short)s.trim().length();
		out.writeShort(count);
		out.writeBytes(s);
	}
	
	void writeRecord(Resource rs, DataOutputStream out, ResourceTableData data) throws Exception
	{

		ResourceTable rt = (ResourceTable) rs;
		Vector vt = data.getRecord();
		
		for (int i=0;i<vt.size();i++)
		{
			switch (rt.getType(i))
			{
			case Resource.SHORT:
				short sv = ((Integer)(vt.elementAt(i))).shortValue();
				out.writeShort(sv);
				out.flush();
				break;
			case Resource.LONG:
				int lv = ((Long)(vt.elementAt(i))).intValue();
				out.writeInt(lv);
				out.flush();
				break;
			case Resource.STRING:
				writeString(out,(String)(vt.elementAt(i)));
				out.flush();
				break;
			case Resource.BOOLEAN:
				boolean bv = ((Boolean)(vt.elementAt(i))).booleanValue();
				if (bv)
					out.writeShort(1);
				else
					out.writeShort(0);
				out.flush();
				break;
			case Resource.FLOAT:
				float fv = ((Float)(vt.elementAt(i))).floatValue();
				out.writeInt(Float.floatToIntBits(fv));
				out.flush();
				break;
			case Resource.DOUBLE:
				float fv0 = ((Double)(vt.elementAt(i))).floatValue();
				out.writeInt(Float.floatToIntBits(fv0));
				out.flush();
				break;
			default:
				break;
			}
		}
	}
}
