/*
 * @(#) MonipulateServer.java 99/3/22
 *
 * Copyright 1997, 1998, 1999 by SRG, Dept. of Computer Science, Hong Kong Univ.
 * Pokfulam RD, Hong Kong, China
 * All rights reserved.
 * 
 * This software is the condifential and prprietary information of SRG. 
 * You shall not disclose such confidential information and shall use it
 * only in accordance with the terms of the license agreement you got this
 * software.
 */

package CLUSTERPROBE.Server;
 
import java.util.EventListener;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Calendar;
import java.net.Socket;
import java.io.IOException;
import CLUSTERPROBE.message.*;
import CLUSTERPROBE.manipulate.*;
import CLUSTERPROBE.resources.Util;
import CLUSTERPROBE.resources.Resource;

/**
 * This class acts as a server for processing packages concerned operations.
 *
 * @author Jimmy Liang
 * @version 1.00 3/22/99
 */

public class ManipulateServer extends ManageServer
{
	EventHandler ev;
	EventHandler manEv;

	public ManipulateServer(int port) {
		super(port);
		ev = new ManipulateRegistryEventHandler();
		manEv = new EventHandler() {
			public synchronized void handleEvent(JessicEvent e) 
			{
				switch (e.getId())
				{
				case JessicEvent.MANIPULATERESULT:
					ManipulateResultEvent event = (ManipulateResultEvent) e;
					EventTransceiver tran = ManipulateUtil.getTransceiver(event);
					if (tran!=null)
					{
						try {
							tran.sendEvent(e);
						}catch(Exception exp) {
							ManipulateUtil.registryTable.remove(tran);
							break;
						}
						ManipulateUtil.handle(event);
					}
					break;
				case JessicEvent.ERROR:
					if (e instanceof ManipulateErrorEvent) {
						ManipulateErrorEvent et = (ManipulateErrorEvent) e;
						EventTransceiver tran1 = ManipulateUtil.getTransceiver(et);
						if (tran1!=null) 
						{
							try {
								tran1.sendEvent(e);
							}catch(Exception exp1){
								ManipulateUtil.registryTable.remove(tran1);
								break;
							}
							ManipulateUtil.handle(et);
						}
					}
					break;
				}
			}
		};
		ManipulateUtil.manEv = manEv;
	}
	
	public void processConnection(Socket s) {
		try {
			System.out.println("Manipulate Server : Establish a connection with a client");
			EventTransceiver tran = new EventTransceiver(s.getInputStream(),s.getOutputStream());
			tran.addHandler(ev,new ManipulateEvent());
			tran.addHandler(manEv, new ManipulateResultEvent());
			tran.addHandler(manEv, new ManipulateErrorEvent());
		}catch(Exception e) {
			System.out.println("Connect Error:"+e.toString());
		}
		System.out.println("Will be out of ManipulationServer Thread.");
	};

}

class ManipulateRegistryEventHandler implements EventHandler
{

	public synchronized void handleEvent(JessicEvent e)
	{
		EventTransceiver tran = EventTransceiver.current;
		if (!(e instanceof ManipulateEvent))
			return;

		try {
		Vector evObjects = (Vector) ManipulateUtil.registryTable.get(tran);
		ManipulateEvent evt = (ManipulateEvent) e;
		int indicator = evt.getIndicator();
		
		switch(indicator)
		{
		case ManipulateEvent.REGISTER:
			if (evObjects == null) {
				evObjects = new Vector();
				ManipulateUtil.registryTable.put(tran,evObjects);
			}
			if (!evObjects.contains(e))
			{
				evObjects.addElement(e);
			}
			ManipulateUtil.register(evt);
			break;
		case ManipulateEvent.SUSPEND:
		case ManipulateEvent.CANCEL:
		case ManipulateEvent.RESUME:
			if (evObjects!=null)
			{
				Enumeration en = evObjects.elements();
				while (en.hasMoreElements())
				{
					ManipulateEvent obj = (ManipulateEvent) en.nextElement();
					if (ManipulateUtil.isEquals(evt,obj))
					{
						ManipulateUtil.changeStatus(evt);
						if (indicator==ManipulateEvent.CANCEL)
							evObjects.removeElement(obj);
					}
				}
			}
			break;
		}
		}catch(Exception ex) {
			System.out.println("Manipulate Registry Error:"+ex.toString());
		}
	}

}
