/*
 * @(#) MonitorServer.java 99/3/1
 *
 * Copyright 1997, 1998, 1999 by SRG, Dept. of Computer Science, Hong Kong Univ.
 * Pokfulam RD, Hong Kong, China
 * All rights reserved.
 * 
 * This software is the condifential and prprietary information of SRG. 
 * You shall not disclose such confidential information and shall use it
 * only in accordance with the terms of the license agreement you got this
 * software.
 */

package CLUSTERPROBE.Server;
 
import java.util.EventListener;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Calendar;
import java.net.Socket;
import java.io.IOException;
import CLUSTERPROBE.message.EventTransceiver;
import CLUSTERPROBE.message.EventHandler;
import CLUSTERPROBE.message.JessicEvent;
import CLUSTERPROBE.message.MonitorErrorEvent;
import CLUSTERPROBE.message.MonitorRegistryEvent;
import CLUSTERPROBE.message.EventTransceiver;
import CLUSTERPROBE.message.RegisterInfoEvent;
import CLUSTERPROBE.message.ResourceTableData;
import CLUSTERPROBE.monitor.MonitorUtil;
import CLUSTERPROBE.monitor.MonitorThread;
import CLUSTERPROBE.resources.Util;
import CLUSTERPROBE.resources.Resource;
import CLUSTERPROBE.message.MonitorDataEvent;

/**
 * This class acts as a server for processing packages concerned monitoring.
 *
 * @author Jimmy Liang
 * @version 1.00 3/1/99
 */

public class MonitorServer extends ManageServer
{
	EventHandler ev;
	EventHandler monEv;

	public MonitorServer(int port) {
		super(port);
		ev = new MonitorRegistryEventHandler();
		monEv = new EventHandler() {
			public synchronized void handleEvent(JessicEvent e) 
			{
				switch (e.getId())
				{
				case JessicEvent.MONITORDATA:	
					MonitorDataEvent event = (MonitorDataEvent) e;
					EventTransceiver tran = MonitorUtil.getTransceiver(event);

					// deliver to local handler
					Vector localHandlers = Util.getHandlers(event.getResourceName());
					if (localHandlers!=null)
					{
						for (int lhi=0;lhi<localHandlers.size();lhi++)
						{
							EventHandler eh = (EventHandler) localHandlers.elementAt(lhi);
							eh.handleEvent(e);
						}
					}

					if (tran!=null)
					{
						try {
							tran.sendEvent(e);
						}catch(Exception exp) {
							MonitorRegistryEvent reg = MonitorUtil.getRegistryEvent(event);
							if (reg!=null)
							{
								reg.setIndicator(MonitorRegistryEvent.UNREGISTER); 						
								MonitorUtil.unregister(reg);
							}
						}
					}

					break;
				case JessicEvent.ERROR:
					if (e instanceof MonitorErrorEvent) {
						MonitorErrorEvent et = (MonitorErrorEvent) e;
						EventTransceiver tran1 = MonitorUtil.getTransceiver(et);
						if (tran1!=null) 
						{
							try {
								tran1.sendEvent(e);
							}catch(Exception exp1){
							MonitorRegistryEvent reg = MonitorUtil.getRegistryEvent(et);
							if (reg!=null)
							{
								reg.setIndicator(MonitorRegistryEvent.UNREGISTER); 						
								MonitorUtil.unregister(reg);
							}
							}
						}
					}
					break;
				}
			}
		};
		MonitorThread.monEv = monEv;
	}
	
	public void processConnection(Socket s) {
		try {
			System.out.println("Monitor Server:Establish a connection with a client");
			EventTransceiver tran = new EventTransceiver(s.getInputStream(),s.getOutputStream());
			tran.addHandler(ev,new MonitorRegistryEvent());
			tran.addHandler(monEv, new MonitorDataEvent());
			tran.addHandler(monEv, new MonitorErrorEvent());
		}catch(Exception e) {
			System.out.println("Connect Error:"+e.toString());
		}
		System.out.println("will be out of MonitorServer Thread.");
	};

}

class MonitorRegistryEventHandler implements EventHandler
{

	public synchronized void handleEvent(JessicEvent e)
	{
		EventTransceiver tran = EventTransceiver.current;
		if (!(e instanceof MonitorRegistryEvent))
			return;

		try {
		Vector evObjects = (Vector) MonitorUtil.registryTable.get(tran);
		MonitorRegistryEvent evt = (MonitorRegistryEvent) e;
		int indicator = evt.getIndicator();

		switch (indicator)
		{
		case MonitorRegistryEvent.MONITOR_EVENT:
		case MonitorRegistryEvent.MONITOR_DATA:
		case MonitorRegistryEvent.NOREPLY:
			if (evObjects == null) {
				evObjects = new Vector();
				MonitorUtil.registryTable.put(tran,evObjects);
			}
			if (!evObjects.contains(e))
			{
				evObjects.addElement(e);
			}
			if (!MonitorUtil.register(evt,tran.getOutStream()))
				evObjects.removeElement(e);
			break;
		case MonitorRegistryEvent.UPDATE:
			if (evObjects!=null)
			{
				Enumeration en = evObjects.elements();
				while(en.hasMoreElements())
				{
					MonitorRegistryEvent obj = (MonitorRegistryEvent) en.nextElement();
					if (MonitorUtil.isEquals(evt,obj))
					{
						evObjects.removeElement(obj);
						evt.setIndicator(obj.getIndicator());
						if (MonitorUtil.update(evt,obj))
							evObjects.addElement(evt);
					}
				}

			}

			if (evObjects.size()==0)
				MonitorUtil.registryTable.remove(tran);
			break;
		case MonitorRegistryEvent.UNREGISTER:
			if (evObjects!=null)
			{
				Enumeration en = evObjects.elements();
				while(en.hasMoreElements())
				{
					MonitorRegistryEvent obj = (MonitorRegistryEvent) en.nextElement();
					if (MonitorUtil.isEquals(evt,obj))
					{
						evObjects.removeElement(obj);
						MonitorUtil.unregister(obj);
					}
				}
				if (evObjects.size()==0)
					MonitorUtil.registryTable.remove(tran);
			}
			break;
		case MonitorRegistryEvent.QUERY:
			sendMonitorInfo(tran);
			break;
		}
						
		}catch(Exception ex) {
			System.out.println("Monitor Registry Error:"+ex.toString());
			ex.printStackTrace();
		}
	} 

	public void sendMonitorInfo(EventTransceiver tran) {
		try {
			Vector evObjects = (Vector) MonitorUtil.registryTable.get(tran);
			if (evObjects==null)
				evObjects = new Vector();
			RegisterInfoEvent reg = new RegisterInfoEvent(RegisterInfoEvent.MONITOR,evObjects);
			tran.sendEvent((JessicEvent)reg);
		}catch(Exception ex) {
			System.out.println("Send monitor information error: "+ex.toString());
		}
	}
		
}
