/*
 * @(#) ResourceServer.java 99/1/6
 */

package CLUSTERPROBE.Server;
 
import java.util.EventListener;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Calendar;
import java.net.Socket;
import java.io.IOException;
import CLUSTERPROBE.message.EventTransceiver;
import CLUSTERPROBE.message.ResourceRefreshEvent;
import CLUSTERPROBE.message.EventHandler;
import CLUSTERPROBE.message.JessicEvent;
import CLUSTERPROBE.message.ResourceRegistryEvent;
import CLUSTERPROBE.message.EventTransceiver;
import CLUSTERPROBE.message.ResourceTableData;
import CLUSTERPROBE.resources.Util;
import CLUSTERPROBE.resources.Resource;
import CLUSTERPROBE.resources.Cluster_NodeLists;

/**
 * This class acts as a server for processing packages concerned resource.
 *
 * @author Jimmy Liang
 * @version 1.00 1/7/99
 */

public class ResourceServer extends ManageServer
{
	EventHandler ev;
	boolean DebugFlag = true;

	public ResourceServer(int port) {
		super(port);
		ev = new ResourceRegistryEventHandler();
		addLocalObservers();
	}
	
	public void addLocalObservers()
	{
		String[] rns = {"CLUSTERPROBE.resources.Cluster_NodeLists","CLUSTERPROBE.resources.Cluster_GroupLists"};

		EventHandler h = new EventHandler() 
		{
			public synchronized void handleEvent(JessicEvent evt) 
			{
				if(DebugFlag) System.out.print("Get JessicEvent:");
				if (evt.getId()==JessicEvent.REFRESH)
				{
					if(DebugFlag) System.out.println("get JessicEvent.REFRESH");
					try {
					ResourceRefreshEvent event = (ResourceRefreshEvent) evt;
					Vector vt = event.getData("CLUSTERPROBE.resources.Cluster_NodeLists");
					if (vt!=null) 
					{
						Vector names = new Vector();
						Enumeration en = vt.elements();
						while (en.hasMoreElements())
						{
							ResourceTableData data = (ResourceTableData) en.nextElement();
							Vector entry = data.getRecord();
							try {
							int ntype = ((Integer)entry.elementAt(4)).intValue();
							if (ntype==Cluster_NodeLists.WORKSTATION)
							{
								names.addElement(((String)entry.elementAt(0)));
								String gn = (String) entry.elementAt(2);
								Vector gh = new Vector();
								if (!Resource.groupToHost.containsKey(gn))
									Resource.groupToHost.put(gn,gh);
								else
									gh = (Vector) Resource.groupToHost.get(gn);
								gh.addElement(((String)entry.elementAt(0)));
							}
							}catch(Exception eee){};
						}
						if (names.size()>0)
						{
							Resource.nodeLists = new String[names.size()];
							Enumeration ns = names.elements();
							int i = 0;
							while (ns.hasMoreElements())
								Resource.nodeLists[i++] = (String) ns.nextElement();
						}

						System.out.println("Resource Lists:");
						for (int ii=0;ii<Resource.nodeLists.length;ii++)
							System.out.println(Resource.nodeLists[ii]+" ");
						System.out.println("");
					}

					// for group lists
					vt = event.getData("CLUSTERPROBE.resources.Cluster_GroupLists");
					if(vt!=null) {
						Resource.qservers = new Vector();
						Enumeration en = vt.elements();
						while (en.hasMoreElements())
						{
							ResourceTableData data = (ResourceTableData) en.nextElement();
							Vector entry = data.getRecord();
							String desc = (String) entry.elementAt(1);
							if (desc!=null&&desc.startsWith("DQS"))
								Resource.qservers.addElement(((String)entry.elementAt(2)));	
						}
						System.out.println("Group lists");
						for (int ii=0;ii<Resource.qservers.size();ii++)
							System.out.println(Resource.qservers.elementAt(ii)+" ");
						System.out.println("");
					}

					}catch(Exception e) {
						System.out.println("Error when handling local events:"+e);
					}
				}
				else if (evt.getId()==JessicEvent.ERROR) {
					if(DebugFlag) System.out.println("get JessicEvent.ERROR");
				};
			}
		};

		for (int i=0;i<rns.length;i++)
		{
			try {
			Resource rs = null;
			if (!Util.contains(rns[i]))
			{
				rs = (Resource) Util.instanceClass(rns[i]);
				System.out.println("1...........................Start a resource Thread:"+rns[i]);
				rs.start();
				Util.addResource(rs);
			}
			else
			{
				rs = Util.getResource(rns[i]);
				Util.addRegister(rns[i]);
			}
								
			Util.addLocalHandler(h,rns[i]); 

			Vector vcts = rs.packRefreshedValue();
			Enumeration events = vcts.elements();
			while (events.hasMoreElements())
				h.handleEvent(((JessicEvent)events.nextElement()));	

			}catch(Exception exp) {
				System.out.println("Error while adding local observers for "+rns[i]+" :"+exp);
			}
		}
	}
			

	public void processConnection(Socket s) {
		try {
			System.out.println("ResourceServer:Establish a connection with a client");
			EventTransceiver tran = new EventTransceiver(s.getInputStream(),s.getOutputStream());
			tran.addHandler(ev,new ResourceRegistryEvent());
		}catch(Exception e) {
			System.out.println("Connect Error:"+e.toString());
		}
		System.out.println("Will be out of ResouceServer Thread.");
	};

}

class ResourceRegistryEventHandler implements EventHandler
{
	private boolean DebugFlag = true;
	
	public synchronized void handleEvent(JessicEvent e)
	{
		if (!(e instanceof ResourceRegistryEvent))
			return;

		try {
		Vector evObjects = (Vector) Util.registryTable.get(EventTransceiver.current);
		ResourceRegistryEvent evt = (ResourceRegistryEvent) e;
		int type = evt.getType();

		switch (type)
		{
		case ResourceRegistryEvent.REGISTER:
//System.out.println("ADD OBSERVER FOR "+evt.getName());
			if(DebugFlag) System.out.println("get ResourceRegistryEvent.REGISTER");
			if (evObjects == null) {
				evObjects = new Vector();
				Util.registryTable.put(EventTransceiver.current,evObjects);
			}
			if (!evObjects.contains(e))
			{
				evObjects.addElement(e);
				sendInitValue((ResourceRegistryEvent) e);
			}
			break;
		case ResourceRegistryEvent.UNREGISTER:
			if(DebugFlag) System.out.println("get ResourceRegistryEvent.UNREGISTER");
			if (evObjects!=null)
			{
				Enumeration en = evObjects.elements();
				while(en.hasMoreElements())
				{
					ResourceRegistryEvent obj = (ResourceRegistryEvent) en.nextElement();
					if (obj.getName().equals(evt.getName())||evt.getName().equals("ALL_RESOURCES"))
					{
//System.out.println("REMOVE OBSERVER FOR "+obj.getName());
						evObjects.removeElement(obj);
						Util.removeRegister(obj.getName());
					}
				}

				if (evObjects.size()==0)
				{
//System.out.println("REMOVE THE TRANSCEIVER FROM TABLE");
					Util.registryTable.remove(EventTransceiver.current);
				}
			}
			break;
		case ResourceRegistryEvent.GETCURRENTVALUE:
			if(DebugFlag) System.out.println("get ResourceRegistryEvent.GETCURRENTVALUE");
			sendCurrentValue(evt.getName());
			//if(DebugFlag) System.out.println("end of sendCurrentValue in ResourceRegistryEvent");
			break;
		case ResourceRegistryEvent.UPDATED:
			if(DebugFlag) System.out.println("get ResourceRegistryEvent.UPDATED");
			updateValue(evt.getName());
			break;
		}
						
		}catch(Exception ex) {
			System.out.println("Resource Registry Error:"+ex.toString());
		}
	} 

	public void sendInitValue(ResourceRegistryEvent e)
	{
		String rn = e.getName();
		
	try {
		// here to startup resource object	
		Resource rs;

		if (!Util.contains(rn))
		{
			System.out.println("instance class :"+rn);
			rs = (Resource) Util.instanceClass(rn);
			System.out.println("...........................Start a resource Thread:"+rn);
			rs.start();	
			Util.addResource(rs);
		}	
		else
		{
			rs = Util.getResource(rn);
			Util.addRegister(rn);
		}

		Vector v = rs.packRefreshedValue();

		// check if local handler observer the resource
		Vector handlers = Util.getHandlers(rn);
		if (handlers!=null&&v!=null) {
			Enumeration evts = v.elements();
			while (evts.hasMoreElements())
			{
				JessicEvent eo = (JessicEvent) evts.nextElement();
				Enumeration hds = handlers.elements();
				while (hds.hasMoreElements())
				{
					EventHandler handler = (EventHandler) hds.nextElement();
					handler.handleEvent(eo);
				}
			}
		}
	

		Enumeration events = v.elements();
		while (events.hasMoreElements())
			EventTransceiver.current.sendEvent((JessicEvent) events.nextElement());
	}catch(Exception ex) {
		System.out.println("Getting Initiate value of resource "+rn+" ocurres error:"+ex.toString());
	};

	}

	public void sendCurrentValue(String rn)
	{
		
	try {
		// here to startup resource object	
		Resource rs;

		if (!Util.contains(rn))
		{
			rs = (Resource) Util.instanceClass(rn);
		}	
		else
		{
			rs = Util.getResource(rn);
		}

		Vector v = rs.packRefreshedValue();
		Enumeration events = v.elements();

		while (events.hasMoreElements()){
			EventTransceiver.current.sendEvent((JessicEvent) events.nextElement());
			System.out.println("SendEvent:(JessicEvent) events");
		}
	}catch(Exception ex) {
		System.out.println("Getting current value of resource "+rn+" ocurres error:"+ex.toString());
		ex.printStackTrace();
	};
	}	

	public void updateValue(String rn)
	{
		
		// here to startup resource object	
		Resource rs;

		try {
		if (!Util.contains(rn))
		{
			rs = (Resource) Util.instanceClass(rn);
		}	
		else
		{
			rs = Util.getResource(rn);
		}
		}catch(Exception e0){
			System.out.println("Error while get resource object:"+e0);
			return;
		}

		Vector v = rs.packRefreshedValue();
		if (v==null)
		{
			return;
		}
		Enumeration events = v.elements();

		try {
		// Send events to local observers
		Vector hds = Util.getHandlers(rn);
		if (hds!=null)
		{
			while (events.hasMoreElements())
			{
				JessicEvent eo = (JessicEvent) events.nextElement();
				Enumeration handlers = hds.elements();
				while (handlers.hasMoreElements())
				{
					EventHandler handler = (EventHandler) handlers.nextElement();
					handler.handleEvent(eo);
				}
			}
		}
		}catch(Exception e1) {
			System.out.println("Error while sending events to local observers:"+e1);
		};

		// Send events to remote observers
		try {
		Enumeration en = Util.registryTable.keys();
		while (en.hasMoreElements())
		{
			EventTransceiver transceiver = (EventTransceiver) en.nextElement();
			Vector vt = (Vector) Util.registryTable.get(transceiver);
			Enumeration registers = vt.elements();
			while (registers.hasMoreElements())
			{
				ResourceRegistryEvent reg = (ResourceRegistryEvent) registers.nextElement();
				if (rn.equals(reg.getName()))
				{
					try {
					transceiver.sendEvents(v);
					break;
					}catch(IOException ex) {
						Util.registryTable.remove(transceiver);
						Util.removeRegister(reg.getName());
					}
				}
			}
		}
		}catch(Exception e2) {
			System.out.println("Error while sending events to remote observers"+e2);
		}

		rs.setRefreshedTime(Calendar.getInstance().getTime());
	}	

	
}
