/*
 * @(#) ManipulateFile.java 99/3/28
 */

package CLUSTERPROBE.manipulate;
 
import java.util.*;
import java.io.*;
import CLUSTERPROBE.message.*;

/**
 * Tools to resume the preset maniplations from file or save new manipulations
 * specify files.
 */

public class ManipulateFile extends Object 
{
	// restore the preset manipulations from file or inputstream
	public static Hashtable getPreset(String f)
	{
		return getPreset(new File(f));
	}

	public static Hashtable getPreset(File f)
	{
		try {
		FileInputStream in = new FileInputStream(f); 
		BufferedReader reader = new BufferedReader(new InputStreamReader(in));
		if (reader==null) return null;
		return getPreset(reader);
		}catch(Exception e) {
			System.out.println(e);
			return null;
		}
	}

	public static Hashtable getPreset(InputStream in)
	{
		try {
		BufferedReader reader = new BufferedReader(new InputStreamReader(in));
		if (reader==null) return null;
		return getPreset(reader);
		}catch(Exception e) {
			System.out.println(e);
			return null;
		}
	}
	
	public static Hashtable getPreset(BufferedReader reader)
	{
		Hashtable ht = new Hashtable();
		try {
		while (true)
		{
			String line = reader.readLine();
			if (line == null) break;
			line = line.trim();
			if (line.startsWith("#")) continue;

			if (line.startsWith("Section"))
			{
				if (line.length()<=7) continue;
				String name = getValue(line.substring(7,line.length()).trim());
				if (name==null||name.equals(""))
				{
					System.out.println("You must define the name of section!");
					continue;
				}

				if (ht.containsKey(name))
				{
					System.out.println("Multiply define "+name);
				}

				ManipulateEvent evt = new ManipulateEvent();
				evt.setName(name);
				ht.put(name,evt);
				getOpt(reader,evt);
			}
		}
		}catch(Exception e) {
			System.out.println(e);
		}
		return ht;
	}

	public static void getOpt(BufferedReader reader, ManipulateEvent evt)
	{
		try {	
		while (true)
		{
			String line = reader.readLine();
			if (line==null) break;
			line = line.trim();
			if (line.startsWith("#")) continue;

			if (line.startsWith("Subsection"))
			{
				if (line.length()<=10) continue;
				String name = getValue(line.substring(10,line.length()).trim());
				if (name==null||name.equals(""))
				{
					System.out.println("You must define the name of subsection!");
					continue;
				}

				getSubsection(reader,name,evt);
			}
			
			if (line.startsWith("Tolerant"))
			{
				if (line.length()<=8) continue;
				String opt = getValue(line.substring(8,line.length()).trim());
				if (opt==null)
				{
					System.out.println("You must set a value for tolerant option!");
					continue;
				}
				if (opt.toLowerCase().equals("true"))
					evt.setForce(true);
				else
					evt.setForce(false);
			}

			if (line.startsWith("Wait"))
			{
				if (line.length()<=4) continue;
				String opt = getValue(line.substring(4,line.length()).trim());
				if (opt==null)
				{
					System.out.println("You must set a value for wait option!");
					continue;
				}
				if (opt.toLowerCase().equals("true"))
					evt.setWait(true);
				else
					evt.setWait(false);
			}

			if (line.startsWith("Confirm"))
			{
				if (line.length()<=7) continue;
				String opt = getValue(line.substring(7,line.length()).trim());
				if (opt==null)
				{
					System.out.println("You must set a value for confirming!");
					continue;
				}
				evt.setConfirmString(opt);
			}

			if (line.startsWith("EndSection"))
					break;
		}
		}catch(Exception e){
			System.out.println(e); 
		}
	}

	public static void getSubsection(BufferedReader reader, String name, ManipulateEvent evt)
	{
		try {
		if (name.equals("Targets"))
		{
			int priority = ManipulateEvent.MAX_PRIORITY;
			while (true)
			{
				String line = reader.readLine();
				if (line==null) break;
				line = line.trim();
				if (line.startsWith("#")) continue;
				if (line.startsWith("EndSubsection"))
					break;

				Vector tempVector = getMultiValue(line);
				if (tempVector.size()==0)
					continue;	
				if (tempVector.size()>1)
					evt.addOrderDesc(priority,(String)tempVector.elementAt(1));

				StringTokenizer sts = new StringTokenizer(line,";");
				line = sts.nextToken().trim();
				Vector hosts = getMultiValue(line," ");
				if (hosts==null||hosts.size()==0)
					continue;
				evt.addOrder(priority++,hosts);
			}
		}
		else if (name.equals("Arguments"))
		{
			Hashtable args = new Hashtable();
			evt.setArguments(args);
			while (true)
			{
				String line = reader.readLine();
				if (line==null) break;
				line = line.trim();
				if (line.startsWith("#")) continue;
				if (line.startsWith("EndSubsection"))
					break;

				Vector vt = getMultiValue(line);
				if (vt.size()!=3) 
					continue;
				String key = (String) vt.elementAt(0);
				if (!key.startsWith("$"))
					continue;
				String desc = (String) vt.elementAt(1);
				String defaultValue = (String) vt.elementAt(2); 
				args.put(key,new ManipulateArgument(desc,defaultValue));
			}
		}
		else if (name.equals("Scripts"))
		{
			int priority = ManipulateEvent.MAX_PRIORITY;
			Hashtable ht = new Hashtable();

			while (true)
			{
				String line = reader.readLine();
				if (line==null) break;
				line = line.trim();
				if (line.startsWith("#")) continue;
				if (line.startsWith("EndSubsection"))
					break;
				
				Vector scripts = getMultiValue(line);
				Vector hosts = evt.getOrder(priority++);
				String prevScript =  null;

				if (hosts==null) 
					continue;
				
				for (int i=0;i<hosts.size();i++)
				{
					String host = (String) hosts.elementAt(i);
					if (scripts.size()>i)		
					{
						String script = (String) scripts.elementAt(i);
						if (script!=null&&script.length()!=0)
						{
							ht.put(host,new ScriptManipulation(script));
							prevScript = script;
						}
						else if (prevScript!=null)
							ht.put(host,new ScriptManipulation(prevScript));
					}
					else
						if (prevScript!=null)
							ht.put(host,new ScriptManipulation(prevScript));
				}
			}

			evt.setOperations(ht);
		}
		}catch(Exception e)
		{
			System.out.println("Parse subsection error:"+e.toString());
		}
	}

	public static String getValue(String s)
	{
		if (!s.startsWith("\"")) 
			return s;
		
		StringTokenizer st = new StringTokenizer(s,"\"");
		if (st.countTokens()==0) return null;
		return st.nextToken();
	}	 
							
	public static Vector getMultiValue(String s)
	{
		return getMultiValue(s,";");
	}

	public static Vector getMultiValue(String s, String tt)
	{
		StringTokenizer st = new StringTokenizer(s,tt);
		Vector vt = new Vector();

		while (st.hasMoreTokens())
			vt.addElement(getValue(st.nextToken().trim()));
		return vt;
	}

	public static void main(String[] args)
	{
		String f = "/etc/presets";
		if (args.length>0)
			f = args[0];
		
		Hashtable ht = ManipulateFile.getPreset(f);
		if (ht==null) System.exit(-1);
		Enumeration en = ht.elements();
		while (en.hasMoreElements())
		{
			ManipulateEvent evt = (ManipulateEvent) en.nextElement();
			Hashtable orders = evt.getOrders();
			System.out.println(orders);
			System.out.println("");
			Hashtable targets = evt.getOperations();
			System.out.println(targets);
			Hashtable orderDesc = evt.getOrderDesc();
			System.out.println(orderDesc);
			System.out.println(evt.getConfirmString());
			System.out.println("-----------------------------");
		}
	}
}
			
