/*
 * @(#) EventTransceiver.java 98/11/23
 */

package CLUSTERPROBE.message;
 
import java.util.*;
import java.net.*;
import java.io.*;


/**
 * This class connect itself to a single remote server to exchange events
 * with it. 
 */

public class EventTransceiver extends Thread implements EventHandler
{

	// A hashtable of handlers for specific events
	private Hashtable handlers = new Hashtable();
	// A list of handlers that want all events
	private Vector globalHandlers = new Vector();

	//Current transciever
	public static EventTransceiver current;
	public static boolean processed = false;

	// Our connection to a remote agent
	InputStream evIn = null;
	OutputStream evOut = null;

	// Establish a connect by host and port
	public EventTransceiver(String host, int port) throws Exception
	{
		InetAddress a = InetAddress.getByName(host);
		connect(a,port);
		current = this;
	}

	public EventTransceiver(InetAddress a, int port) throws Exception
	{
		connect(a,port);
		current = this;
	}

	public EventTransceiver(InputStream in, OutputStream out) {
		setStreams(in,out);
		current = this;
		System.out.println("...........................Start Event Transceiver Thread ");
		start();
	}

	public OutputStream getOutStream()
	{
		return evOut;
	}

	// conncet to server
	void connect(InetAddress a, int port) throws Exception
	{
		Socket s = new Socket(a,port);
		evIn = s.getInputStream();
		evOut = s.getOutputStream();
		System.out.println("...........................Start Event Transceiver Thread ");
		start();
	}

	// disconnect 
	public void disconnect() {
		try {
		if (evIn!=null) evIn.close();
		if (evOut!=null) evOut.close();
		}catch(Exception e){};
	}

	// No default hehavior for handling events...
	public void handleEvent(JessicEvent e) {
	}

	void setStreams(InputStream in, OutputStream out)
	{
		evIn = in;
		evOut = out;
	}

	// send messages based on event to server
	public void sendEvent(JessicEvent ev) throws IOException {
	//System.out.println("Begin Sending Event");
		ObjectOutputStream out = new ObjectOutputStream(evOut);
		out.writeObject(ev);
	//System.out.println("End Sending Event");
	}

	public void sendEvents(Vector vt ) throws Exception {
		if (vt!=null) {
			Enumeration en = vt.elements();
			while (en.hasMoreElements())
			{
				JessicEvent ev  = (JessicEvent) en.nextElement();
				sendEvent(ev);
			}
		}
	}
		
	// receive the coming events
	JessicEvent receiveEvent() throws IOException, ClassCastException, ClassNotFoundException 
	{
		ObjectInputStream in = new ObjectInputStream(evIn);
		//System.out.println("Receive Event");
		JessicEvent ev = null;
		ev = (JessicEvent) in.readObject();
		System.out.println("Receive Event Object:"+ev);
		return ev;
	}

	// Distributes coming events to corresponding handlers
	void distributeEvent(JessicEvent ev) 
	{
		while (processed) 
			Thread.currentThread().yield();
		processed = true;
		current = this;

		// Send event to all global handlers
		Enumeration e = globalHandlers.elements();
		while (e.hasMoreElements()) {
			EventHandler h = (EventHandler) e.nextElement();
			h.handleEvent(ev);
		}

		// Send event to handlers targeting the event's class
		Class evClass = ev.getClass();
//System.out.println(evClass.getName());
		Vector evHandlers = (Vector) handlers.get(evClass);
		e = evHandlers.elements();
		while (e.hasMoreElements()) {
			EventHandler h = (EventHandler) e.nextElement();
			h.handleEvent(ev);
		}
		processed = false;
	}

	// Register a handler that observing all events
	public void addHandler (EventHandler eh) 
	{
		if (!globalHandlers.contains(eh))
			globalHandlers.addElement(eh);
	}

	// Register a handler for a especial tpye of event
	public void addHandler(EventHandler eh, JessicEvent e) 
	{
		Class evClass = e.getClass();
		addHandler(eh,evClass);
	}

	public void addHandler(EventHandler eh, Class c)
	{
		Vector evHandlers = (Vector) handlers.get(c);
		if (evHandlers==null) {
			evHandlers = new Vector();
			handlers.put(c,evHandlers);
		}
		if (!evHandlers.contains(eh))
			evHandlers.addElement(eh);
	}

	//Remove a handler from global and specific handlers table
	public void removeHandler(EventHandler eh) {
		globalHandlers.removeElement(eh);
		Enumeration eList = handlers.keys();
		while (eList.hasMoreElements())
		{
			Vector evHandlers = (Vector) handlers.get(eList.nextElement());
			if (evHandlers!=null) {
				evHandlers.removeElement(eh);
			}
		}
	}

	// Remove a handler for specific event type
	public void removeHandler(EventHandler eh, JessicEvent e) {
		removeHandler(eh,e.getClass());
	}

	public void removeHandler(EventHandler eh, Class c) 
	{
		Vector evHandlers = (Vector) handlers.get(c);
		if (evHandlers != null)
			evHandlers.removeElement(eh);
	}

	/** 
	 * If run as an independent thread, just loop listening for event
	 * from the remote server, then distribute them to registerred 
	 * handlers.
	 **/
	public void run() {
		try {
			while (true) {
				try {
					JessicEvent e = receiveEvent();
				}catch(IOException e){
					System.out.println("Receiving thread die in while:"+e.toString());
				}
				try{
				if (e!=null)
					distributeEvent(e);
				}
				catch(ClassCastException e) {
					System.out.println("Non-event object sent to EventTransceiver");
				}
				catch(ClassNotFoundException e) {
					System.out.println("Unresolvable object type sent to EventTransceiver");
				}
			}
		}
		// Treat an IOException when receiving events as termination of
		// input stream, and let this thread die
		catch(IOException e) {
			System.out.println("Receiving thread die: "+e.toString());
			//current=null;
		}
	}
}
