/*
 * @(#) ManipulateEvent.java 99/3/22
 *
 * Copyright 1997, 1998 by SRG, Dept. of Computer Science, Hong Kong Univ.
 * Pokfulam RD, Hong Kong, China
 * All rights reserved.
 * 
 * This software is the condifential and prprietary information of SRG. 
 * You shall not disclose such confidential information and shall use it
 * only in accordance with the terms of the license agreement you got this
 * software.
 */

package CLUSTERPROBE.message;
 
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Calendar;
import CLUSTERPROBE.manipulate.*;
import CLUSTERPROBE.security.Password;

/**
 * The event type defined by this class stands for operation message from
 * clients.
 *
 * @author Jimmy Liang
 * @version 1.00 3/22/99
 */

public class ManipulateEvent extends JessicEvent
{
	String operationName = null;
	String desc = null;
	String confirmString =  null;
	long sendOperateTime = 0;
	int indicator = 0;

	Password pw = null;

	public final static int MAX_PRIORITY = 0;
	public final static int MIN_PRIORITY = 256;

	public final static int REGISTER = 0;
	public final static int CANCEL = 1;
	public final static int SUSPEND = 2;
	public final static int RESUME = 3;

	// Refer to the targets and the relative operations which will be executed on target nodes
	Hashtable targets = new Hashtable();
	// Refer to the priorities of the executions 
	Hashtable orders = new Hashtable();
	// Descripton for the orders
	Hashtable  orderDesc =  new Hashtable();

	// Store the arguments for operation
	Hashtable args = new Hashtable();

	// whether force execute when errors occur
	boolean tolerance = true;
	// whether do the next operations after the prior operations were completed
	boolean wait = true;

	public ManipulateEvent()
	{
		super("");
		sendOperateTime = Calendar.getInstance().getTime().getTime();
		setId(JessicEvent.MANIPULATEREGISTRY);
	}

	public ManipulateEvent(String name, Hashtable ht)
	{
		super("");
		operationName = name;
		targets = ht;
		sendOperateTime = Calendar.getInstance().getTime().getTime();
		setId(JessicEvent.MANIPULATEREGISTRY);
	}

	public long getTime()
	{
		return sendOperateTime;
	}

	public void setTime(long t)
	{
		sendOperateTime = t;
	}

	public void setIndicator(int type)
	{
		indicator = type;
	}

	public int getIndicator()
	{
		return indicator;
	}

	public void setDesc(String d)
	{
		desc = d;
	}

	public String getDesc()
	{
		return desc;
	}

	public void setName(String n)
	{
		operationName = n;
	}

	public String getName()
	{
		return operationName;
	}

	public void setOperations(Hashtable ht)
	{
		targets = ht;		
	}

	public Hashtable getOperations()
	{
		return targets;
	}

	public ManipulateArgument getArgument(String key)
	{
		return (ManipulateArgument) args.get(key);
	}

	public void addArgument(String key, ManipulateArgument value)
	{
		args.put(key,value);
	}

	public Hashtable getArguments()
	{
		return args;
	}

	public void setArguments(Hashtable ht)
	{
		args = ht;
	}

	public String[] getTargets()
	{
		if (targets==null) return null;
		Enumeration en = targets.keys();
		String[] s = new String[targets.size()];
		for (int i=0;i<targets.size();i++)
			s[i] = (String)en.nextElement();
		return s;
	}

	public void removeTarget(String s)
	{
		if (targets==null) return;
		targets.remove(s);
	}

	public void addTarget(String s, Manipulation m)
	{
		if (targets==null) return;
		if (targets.containsKey(s))
			targets.remove(s);
		targets.put(s,m);
	}

	public void clear()
	{
		if (targets==null) 
			targets = new Hashtable();
		targets.clear();
	}

	public Manipulation getManipulation(String s)
	{
		if (targets==null) return null;
		if (targets.containsKey(s))
			return (Manipulation) targets.get(s);
		return null;
	}

	public Hashtable getOrders()
	{
		return orders;
	}

	public void setOrders(Hashtable ht)
	{
		orders = ht;
	}

	public int[] getPriorities()
	{
		if (orders==null) return null;
		Enumeration en = orders.keys();
		Vector v = new Vector();
		while (en.hasMoreElements())
			v.addElement(en.nextElement());	
		int[] p = new int[v.size()];
		for (int i=0;i<p.length;i++)
			p[i] = ((Integer)v.elementAt(i)).intValue();
		return p;
	}

	public int getMaxPriority()
	{
		int[] p = getPriorities();
		if (p!=null&&p.length>0)
			return p[p.length-1];
		return 0;
	}

	public int getMinPriority()
	{
		int[] p=getPriorities();
		if (p!=null&&p.length>0)
			return p[0];
		return 0;
	}

	public void removeOrder(int order)
	{
		Integer p = new Integer(order);
		if (orders!=null)
			orders.remove(new Integer(order));
	}

	public void clearOrder()
	{
		if (orders==null)
			orders = new Hashtable();
		else
			orders.clear();
	}

	public void addOrder(int o,String t )
	{
		if (orders==null) orders = new Hashtable();
		Vector v = (Vector)orders.get(new Integer(o));
		if (v==null)
		{
			v = new Vector();
			orders.put(new Integer(o),v);
		}
		if (!v.contains(t))
			v.addElement(t);
	}

	public void addOrder(int o, Vector v)
	{
		if (orders==null) orders = new Hashtable();
		if (orders.containsKey(new Integer(o)))
			orders.remove(new Integer(o));
		orders.put(new Integer(o),v);
	}

	public void removeOrder(int o,String s)
	{
		if (orders!=null)
		{
			Vector v = (Vector)orders.get(new Integer(o));
			if (v!=null&&v.contains(s))
				v.removeElement(s);
		}
	}
	
	public Vector getOrder(int o)
	{
		if (orders==null) return null;
		return (Vector) orders.get(new Integer(o));
	}

	public Hashtable getOrderDesc()
	{
		return orderDesc;
	}

	public String getOrderDesc(int o)
	{
		if (orderDesc.containsKey(new Integer(o)))
			return (String)orderDesc.get(new Integer(o));
		return null;
	}

	public void setOrderDesc(Hashtable ht)
	{
		orderDesc = ht;
	}

	public void addOrderDesc(int o, String t)
	{
		if (orderDesc.containsKey(new Integer(o)))
			orderDesc.remove(new Integer(o));
		orderDesc.put(new Integer(o),t);
	}

	public void removeOrderDesc(int o)
	{
		if (orderDesc.containsKey(new Integer(o)))
			orderDesc.remove(new Integer(o));
	}

	public void clearOrderDesc()
	{
		orderDesc = new Hashtable();
	}
			
	public boolean isForce() {
		return tolerance;
	}

	public void setForce(boolean f) {
		tolerance = f;
	}

	public boolean isWait() {
		return wait;
	}

	public void setWait(boolean w) {
		wait = w ;
	}

	public void setPasswd(Password pw)
	{
		this.pw = pw;
	}

	public Password getPasswd()
	{
		return pw;
	}

	public String getConfirmString()
	{
		return confirmString;
	}

	public void setConfirmString(String s)
	{
		confirmString = s;
	}

	public static void main(String[] args)
	{
		Manipulation mp = new Manipulation() {
			public void operate(Hashtable ht, Password pw)
			{};

			public boolean isAlive() {
				return false;
			}
			
			public Object getResult()
			{
				return null;
			}	
		
			public long getTimeout() {
				return 0;
			}

			public boolean isSuccess() {
				return false;
			}
		};

		Hashtable ht = new Hashtable();
		ht.put("one",mp);
		ht.put("two",mp);
		ht.put("three",mp);
		ht.put("four",mp);
		ht.put("five",mp);
		ht.put("six",mp);
		ht.put("seven",mp);

		ManipulateEvent m = new ManipulateEvent("Test",ht);
		Vector v = new Vector();
		v.addElement("one");
		v.addElement("seven");
		m.addOrder(5,v);
		Vector v1 = new Vector();
		v1.addElement("three");
		m.addOrder(2,v1);
		Vector v2 = new Vector();
		v2.addElement("two");
		m.addOrder(3,v2);
		Vector v3 = new Vector();
		v3.addElement("four");
		v3.addElement("five");
		m.addOrder(0,v3);
		Vector v4 = new Vector();
		v4.addElement("six");
		m.addOrder(1,v4);

		System.out.println(m.getOrders());
		System.out.println(m.getMinPriority());
		System.out.println(m.getMaxPriority());
		int[] p = m.getPriorities();
		for (int i=0;i<p.length;i++)
			System.out.print(p[i]+" ");
		System.out.println();
	}


}
