/*
 * @(#) MonitorRegistryEvent.java 99/3/02
 *
 * Copyright 1997, 1998 by SRG, Dept. of Computer Science, Hong Kong Univ.
 * Pokfulam RD, Hong Kong, China
 * All rights reserved.
 * 
 * This software is the condifential and prprietary information of SRG. 
 * You shall not disclose such confidential information and shall use it
 * only in accordance with the terms of the license agreement you got this
 * software.
 */

package CLUSTERPROBE.message;
 
import CLUSTERPROBE.monitor.React;
import CLUSTERPROBE.monitor.Condition;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Calendar;

/**
 * The event type defined by this class stands for monitor resources abnormal
 * changes.
 *
 * @author Jimmy Liang
 * @version 1.00 3/2/99
 */

public class MonitorRegistryEvent extends JessicEvent
{
	// define the indicators	
	public final static int MONITOR_EVENT = 0;
	public final static int MONITOR_DATA = 1;
	public final static int NOREPLY = 2; 
	public final static int UPDATE = 3;
	public final static int QUERY = 4;
	public final static int UNREGISTER = 5;

	// default indicator is that monitor the abnormal events with data 
	int indicator = MONITOR_DATA;

	// refer to the monitored targets 
	String[] sourceNames ;

	// the location of monitored object
	String resourceName = null;
	int[] entry = null;
	Object[] index = null;

	// the interval for monitoring
	int monitorInterval = 2;
	int reportInterval = 1;

	// the condition for occuring abnormal events
	Condition condition = null;

	// the reactions when occuring abnormal events
	React react = null;

	// Register time
	long registerTime = 0;

	public MonitorRegistryEvent() {
		super("");
		setId(JessicEvent.MONITORREGISTRY);
		registerTime = Calendar.getInstance().getTime().getTime();
	}

	public MonitorRegistryEvent(int type) 
	{
		super("");
		setId(JessicEvent.MONITORREGISTRY);
		registerTime = Calendar.getInstance().getTime().getTime();
		indicator = type;
	}

	// set/get indicator
	public void setIndicator(int in) {
		indicator = in;
	}

	public int getIndicator() {
		return indicator;
	}

	// set/get sourceNames
	public void setTargets(String[] s)
	{
		sourceNames = s;
	}

	public String[] getTargets()
	{
		return sourceNames;
	}

	// set/get location of monitored object
	public void setLocation(String rn, int[] en, Object[] in)
	{
		resourceName = rn;
		entry = en;
		index = in;
	}

	public String getResourceName() {
		return resourceName;
	}

	public int[] getEntry() {
		return entry;
	}

	public Object[] getIndex() {
		return index;
	}


	// set/get condition
	public void setCondition(Condition con)
	{
		condition = con;	
	}
	
	public Condition getCondition()
	{
		return condition;
	}

	// set/get reaction
	public void setReaction(React re)
	{
		react = re;
	}

	public React getReaction() {
		return react;
	}

	// set/get interval
	public void setInterval(int m, int r)
	{
		monitorInterval = m;
		reportInterval = r;
	}

	public int getMonitorInterval()
	{
		return monitorInterval;
	}

	public int getReportInterval()
	{
		return reportInterval;
	}

	// get register time
	public long getRegisterTime()
	{
		return registerTime;
	}
}
