/*
 * @(#) ResourceRefreshEvent.java 98/11/23
 *
 * Copyright 1997, 1998 by SRG, Dept. of Computer Science, Hong Kong Univ.
 * Pokfulam RD, Hong Kong, China
 * All rights reserved.
 * 
 * This software is the condifential and prprietary information of SRG. 
 * You shall not disclose such confidential information and shall use it
 * only in accordance with the terms of the license agreement you got this
 * software.
 */

package CLUSTERPROBE.message;
 
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;

/**
 * The event type define by this class stands for cluster resource status  
 * changing , and also embeds the current value of resource status.
 *
 * @author Jimmy Liang
 * @version 1.00 12/21/98
 */

public class ResourceRefreshEvent extends JessicEvent
{
	
	Hashtable refreshResources = new Hashtable();
	long refreshTime;	
	boolean finalOne = true;
	Object location = null;

	public final static int REFRESHVALUES = 0;
	public final static int REFRESHRECORD = 1;

	
	public ResourceRefreshEvent() 
	{
		super("");
		setId(JessicEvent.REFRESH);
	};

	public ResourceRefreshEvent(Object source, long time)
	{
		super("");
		setId(JessicEvent.REFRESH);
		location = source;
		refreshTime = time;
	}

	
	// get and set the time ocurring this event
	public long getRefreshTime()
	{
		return refreshTime;
	}

	public void setRefreshTime(long time)
	{
		refreshTime = time;	
	}

	// get and set the final one mark
	public boolean isFinalOne()
	{
		return finalOne;
	}

	public void setFinalOne(boolean b)
	{
		finalOne = b;
	}

	// get and set the location of resource
	public Object getLocation()
	{
		return location;
	}

	public void setFinalOne(Object l)
	{
		location = l;
	}

	// get all resource names involved in this event
	public String[] getResourceNames()
	{
		if (refreshResources==null) return null;
		Enumeration en = refreshResources.keys();
		if (en==null) return null;	
		int i=0;	
		String[] names = new String[refreshResources.size()];
		while (en.hasMoreElements())
			names[i++] = (String) en.nextElement();	
		return names;
	}

	public int getCount() {
		return refreshResources.size();
	}

	public Vector getData(String rn) {
		return (Vector) refreshResources.get(rn);
	}

	public boolean contains(String rn) {
		return refreshResources.containsKey(rn);
	}

	// add a resource refreshing data 
	public void addResource(ResourceData data)
	{
		String name = data.getName();	
		Vector vt = (Vector) refreshResources.get(name);
		if (vt == null)
		{
			vt = new Vector();
			refreshResources.put(name,vt);
		}
		if (!vt.contains(data))
			vt.addElement(data);
	}	

	// Remove a data
	public void removeResource(ResourceData data) 
	{
		Vector v = getData(data.getName());
		if (v!=null)
			v.removeElement(data);
	}	

}
