/*
 * @(#) Cluster_SystemLog.java 99/4/23
 *
 * Copyright 1997, 1998 by SRG, Dept. of Computer Science, Hong Kong Univ.
 * Pokfulam RD, Hong Kong, China
 * All rights reserved.
 * 
 * This software is the condifential and prprietary information of SRG. 
 * You shall not disclose such confidential information and shall use it
 * only in accordance with the terms of the license agreement you got this
 * software.
 */

package CLUSTERPROBE.resources;
 
import java.util.*;
import java.io.LineNumberReader;
import java.io.InputStreamReader;
import java.io.FileInputStream;

/**
 * Define the resource format of system logs
 *
 * @author Jimmy Liang
 * @version 1.00 4/23/99
 */

public class Cluster_SystemLog extends ResourceTable
{
	String[] entryNames = {"Log ID","Priority","Date","Source","Daemon","Description"};
	int[] entryTypes = {Resource.SHORT,Resource.STRING,Resource.STRING,Resource.STRING,Resource.STRING,Resource.STRING};
		
	int[] entryMaxLen = {6,10,16,16,16,50};

	static int index = 0;

	public Cluster_SystemLog() 
	{
		setUpdateTime(30*1000);
		setMinTime(10*1000);
	}

	public String[] getEntryNames() {
		return entryNames;
	}

	public String getKeyName() {
		return entryNames[0];
	}

	public boolean isValueFromServer() {
		return true;
	}

	public int[] getEntryType() {
		return entryTypes;
	}

	public int getTableType() {
		return ResourceTable.SIGLETABLE;
	}

	public Vector getRange(String entryName)
	{
		return null;
	}

	public Hashtable getTable()
	{
		Hashtable ht = new Hashtable();
		LineNumberReader d = null;

		try {
			FileInputStream in = new FileInputStream("/var/log/messages");
			d = new LineNumberReader(new InputStreamReader(in));

		}catch(Exception e) {
			System.out.println("Error while reading log file:"+e.toString());
			addErrorEvent("Error while reading log file:"+e.toString());
			return null;
		}
		if (d==null) return null;


		try {
		if (index==0) {
			while ((d.readLine())!=null)
				index++;
			return ht;
		}
	
		if (index != -1)
		{
			try {
			for (int ii=0;ii<index;ii++)
				d.readLine();
			}catch(Exception ee) {
				index = -1;
				return ht;
			}
		}

		String line = null;
		int id = 0;
		int rl = 0;

		while ((line=d.readLine())!=null)
		{
			try {
			rl++;
			if (line.startsWith("#")) continue;
			if (line.indexOf(":")==-1) continue;
			StringTokenizer st = new StringTokenizer(line,":");
			StringTokenizer st1 = new StringTokenizer(line);
			int cc = st.countTokens();
			if (cc<4) continue;
			
			String priority = "info";

			String date = "";
			for (int ii=0;ii<3;ii++)
				date = date + " " + st1.nextToken();

			st.nextToken();
			st.nextToken();
			String temp = st.nextToken();
			String description = "";
			for (int ii=3;ii<cc;ii++)
				description = description+" "+st.nextToken();

			String source = st1.nextToken();

			StringTokenizer st2 = new StringTokenizer(temp);
			int count = st2.countTokens();
			if (count<3) continue;
			st2.nextToken();
			st2.nextToken();
			String daemon = "";
			for (int ii=2;ii<count;ii++)
				daemon = daemon + " "+st2.nextToken();

			Vector v = new Vector();
			v.addElement(new Integer(id));
			v.addElement(priority);
			v.addElement(date);
			v.addElement(source);
			v.addElement(daemon);
			v.addElement(description);
			ht.put(new Integer(id++),v);
			}catch(Exception ex){
				addErrorEvent("Error while parsing log file:"+ex.toString());
			};
		}

		index = index + rl;
		}catch(Exception exc){
			System.out.println(exc);
			addErrorEvent("Error while parsing queue.conf file:"+exc.toString());
		}
		try {
			d.close();
		}catch(Exception ee){};

		return ht;
	}

	public int getEntryMaxLen(int id)
	{
		return entryMaxLen[id];
	}

	public static void main(String[] args) {
		Cluster_SystemLog rs = new Cluster_SystemLog();
		Hashtable ht = rs.getTable();
		System.out.println(ht.size());
		System.out.println(ht);
	}

	public String getDesc()
	{
		return "System Log";
	}	

}
