/*
 * @(#) Cluster_Workstation_CPUInfo.java 01/2/23
 */

package CLUSTERPROBE.resources;
 
import java.util.*;
import java.io.DataInputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.FileInputStream;

/**
 * Define the resource format of Cluster-Workstation-CPUInfo
 */

public class Cluster_Workstation_CPUInfo extends ResourceTable
{
	String[] entryNames = {"processor","vendor_id","cpu","model","stepping","fdiv_bug","hlt_bug","fpu","cpuid","wp","bogomips"};

	int[] entryTypes = {Resource.SHORT,Resource.STRING,Resource.STRING,Resource.STRING,Resource.STRING,Resource.BOOLEAN,Resource.BOOLEAN,Resource.BOOLEAN,Resource.BOOLEAN,Resource.BOOLEAN,Resource.FLOAT};

	int[] entryMaxLen = {10,8,8,20,8,9,8,5,7,5,10};
		
	static int UPDATETIME=24*60*60*1000;

	public Cluster_Workstation_CPUInfo()
	{
		setUpdateTime(UPDATETIME);
		setMinTime(2*60*1000);
	}

	public Hashtable getHashtable() throws java.rmi.ServerRuntimeException 
	{
		Hashtable ht = new Hashtable();
		BufferedReader reader = null;

		try {
			FileInputStream in = new FileInputStream("/proc/cpuinfo");
			reader = new BufferedReader(new InputStreamReader(in));
		}catch(Exception ex) {
			System.out.println("Error while reading /proc/cpuinfo file:"+ex.toString());
			throw new java.rmi.ServerRuntimeException("Error while reading /proc/cpuinfo file!", ex); 
		}

		if (reader==null) return null;

		String line = null;
		boolean bb = false;
		Vector vt = null;

		try {
		while (true)
		{
			line = reader.readLine();
			if (line==null) break;

			if (line.startsWith("processor"))
			{
				StringTokenizer st = new StringTokenizer(line,":");
				if (st.countTokens()!=2) continue;
				st.nextToken();
				String sss = st.nextToken();
				Integer id = new Integer(sss.trim());
				ht.put(id,new Vector());
				vt = (Vector) ht.get(id);
				vt.addElement(id);
				bb = true;
				continue;
			}

			if (!bb) continue;

			for (int i=0;i<entryNames.length;i++) {
				String name = entryNames[i];
				if (line.startsWith(name)) {
					int size = vt.size();
					if (size>getId(name)) continue;
					Object obj = null;

					try {
					int type = entryTypes[getId(name)];
					StringTokenizer st = new StringTokenizer(line,":");
					st.nextToken();
					if (type==Resource.SHORT)
						obj = (Object)( new Integer(st.nextToken().trim()));
					else if (type==Resource.STRING)
						obj = (Object) (st.nextToken());
					else if (type==Resource.FLOAT)
						obj = (Object) (new Float(st.nextToken().trim()));
					else if (type==Resource.BOOLEAN) {
						String str = st.nextToken().trim().toLowerCase();
						if (str.equals("yes"))
							obj = (Object) (Boolean.TRUE);
						else
							obj = (Object)(Boolean.FALSE);
					}
						
					}catch(Exception e1) {
						System.out.println("Error while gotting info:"+e1);
					}

					if (size==getId(name))
						vt.addElement(obj);
					else
					{
						for (int ii=0;ii<(getId(name)-size);ii++)
							vt.addElement(null);
						vt.addElement(obj);
					}
					break;
				}
			}
		}
		}catch(Exception e2) {
			System.out.println("Error while gotting info:"+e2);
			e2.printStackTrace();
			return null;
		}
		try {
		reader.close();
		}catch(Exception ee) {};
		return ht; 	
        }

	public String[] getEntryNames() {
		return entryNames;
	}

	public String getKeyName() {
		return entryNames[0];
	}

	public int[] getEntryType() {
		return entryTypes;
	}

	public int getTableType() {
		return ResourceTable.MULTITABLE;
	}

	public Vector getRange(String entryName)
	{
		return null;
	}

	public int getEntryMaxLen(int id)
	{
		return entryMaxLen[id];
	}

	public static void main(String[] args) {
		Cluster_Workstation_CPUInfo rs = new Cluster_Workstation_CPUInfo();
		try {
		Hashtable ht = rs.getHashtable();
		if (ht==null) 
		{
			System.out.println("null");
			System.exit(-1);
		}
		Enumeration en = ht.keys();
		while (en.hasMoreElements())
		{
			Object obj = en.nextElement();
			Vector vt = (Vector) ht.get(obj);
			System.out.println("size:"+vt.size());
			for (int i=0;i<vt.size();i++)
			{
				Object oo = vt.elementAt(i);
				System.out.println(i+":"+oo);
			}
		}
		}catch(Exception e) {System.out.println(e);}
		System.exit(-1);
	}

	public String getDesc()
	{
		return "CPU Information";
	}	
}
