/*
 * @(#) Cluster_Workstation_CPUState.java 99/4/24
 *
 * Copyright 1997, 1998 by SRG, Dept. of Computer Science, Hong Kong Univ.
 * Pokfulam RD, Hong Kong, China
 * All rights reserved.
 * 
 * This software is the condifential and prprietary information of SRG. 
 * You shall not disclose such confidential information and shall use it
 * only in accordance with the terms of the license agreement you got this
 * software.
 */

package CLUSTERPROBE.resources;
 
import java.util.*;
import java.io.DataInputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.FileInputStream;

/**
 * Define the resource format of Cluster-Workstation-CPU State
 *
 * @author Jimmy Liang
 * @version 1.00 04/23/99
 */

public class Cluster_Workstation_CPUState extends ResourceTable
{
	String[] entryNames = {"CPU Name","User","System","Nice","Idle"};
	int[] entryTypes = {Resource.STRING,Resource.LONG,Resource.LONG,Resource.LONG,Resource.LONG};
	int[] entryMaxLen = {8,10,10,10,10};

	String fn = "/proc/stat";

	static int UPDATETIME=10*1000;

	Hashtable p_user  = new Hashtable();
	Hashtable p_system  = new Hashtable();
	Hashtable p_nice  = new Hashtable();
	Hashtable p_idle  = new Hashtable();

	public Cluster_Workstation_CPUState()
	{
		setUpdateTime(UPDATETIME);
		setMinTime(5*100);
	}

	public Hashtable getHashtable() throws java.rmi.ServerRuntimeException 
	{
		Hashtable ht = new Hashtable();
		BufferedReader reader = null;

		try {
			FileInputStream in = new FileInputStream(fn);
			reader = new BufferedReader(new InputStreamReader(in));
		}catch(Exception ex) {
			System.out.println("Error while reading file "+fn+ex.toString());
			throw new java.rmi.ServerRuntimeException("Error while reading file "+fn, ex); 
		}

		if (reader==null) return null;

		String line = null;

		try {
		while (true)
		{
			line = reader.readLine();
			if (line==null) break;

			if (line.startsWith("#")) continue;
			if (!line.startsWith("cpu")) continue;

			StringTokenizer st = new StringTokenizer(line);
			int count = st.countTokens();
			if (count!=5) continue;

			String cpuName = st.nextToken();
			long tempData = 0;

			long tv = Long.parseLong(st.nextToken());
			tempData = getData(p_user,cpuName,tv);
			long user = tv - tempData;
			if (user<0) user = tv;

			tv = Long.parseLong(st.nextToken());
			tempData = getData(p_system,cpuName,tv);
			long system = tv - tempData;
			if (system<0) system = 0;

			tv = Long.parseLong(st.nextToken());
			tempData = getData(p_nice,cpuName,tv);
			long nice = tv - tempData;
			if (nice<0) nice = tv;

			tv = Long.parseLong(st.nextToken());
			tempData = getData(p_idle,cpuName,tv);
			long idle = tv - tempData;
			if (idle<0) idle = tv;

			Vector vt = new Vector();
			vt.addElement(cpuName);
			vt.addElement(new Long(user));
			vt.addElement(new Long(system));
			vt.addElement(new Long(nice));
			vt.addElement(new Long(idle));
			ht.put(cpuName,vt);
		}
		}catch(Exception e2) {
			System.out.println("Error while gotting info:"+e2);
			e2.printStackTrace();
		}
		try {
			reader.close();
		}catch(Exception ee){};
		return ht; 	
        }

	public String[] getEntryNames() {
		return entryNames;
	}

	public String getKeyName() {
		return entryNames[0];
	}

	public int[] getEntryType() {
		return entryTypes;
	}

	public int getTableType() {
		return ResourceTable.MULTITABLE;
	}

	public Vector getRange(String entryName)
	{
		return null;
	}

	public int getEntryMaxLen(int id)
	{
		return entryMaxLen[id];
	}

	public static void main(String[] args) {
		Cluster_Workstation_CPUState rs = new Cluster_Workstation_CPUState();
		try {
		Hashtable ht = rs.getHashtable();
		if (ht==null) 
		{
			System.out.println("null");
			System.exit(-1);
		}
		System.out.println(ht);
		Thread.sleep(1000);
		ht = rs.getHashtable();
		System.out.println(ht);
		}catch(Exception e) {System.out.println(e);}
		System.exit(-1);
	}

	public long getData(Hashtable ht, String key, long newValue)
	{
		if (ht.containsKey(key))
		{
			long value = ((Long)ht.get(key)).longValue();
			ht.put(key,new Long(newValue));
			return value;
		}
		else
		{
			ht.put(key,new Long(newValue));
			return 0;
		}
	}

	public String getDesc()
	{
		return "CPU Statistics";
	}
}
