/*
 * @(#) Cluster_Workstation_MemState.java 98/4/23
 *
 * Copyright 1997, 1998 by SRG, Dept. of Computer Science, Hong Kong Univ.
 * Pokfulam RD, Hong Kong, China
 * All rights reserved.
 * 
 * This software is the condifential and prprietary information of SRG. 
 * You shall not disclose such confidential information and shall use it
 * only in accordance with the terms of the license agreement you got this
 * software.
 */

package CLUSTERPROBE.resources;
 
import java.util.*;
import java.io.DataInputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.FileInputStream;

/**
 * Define the resource format of Cluster-Workstation-Memory State
 *
 * @author Jimmy Liang
 * @version 1.00 12/23/98
 */

public class Cluster_Workstation_MemState extends ResourceTable
{
	String[] entryNames = {"Host Name","MemTotal","MemUsed","MemFree","MemShared","Buffers","Cached"};
	int[] entryTypes = {Resource.STRING,Resource.LONG,Resource.LONG,Resource.LONG,Resource.LONG,Resource.LONG,Resource.LONG};
	int[] entryMaxLen = {15,10,10,10,10,10,10};
		
	static int UPDATETIME=10*1000;
	static String fn = "/proc/meminfo";

	public Cluster_Workstation_MemState() {
		setUpdateTime(UPDATETIME);
		setMinTime(5*1000);
	}

	public Vector getVector() throws java.rmi.ServerRuntimeException
	{
		BufferedReader d = null;

		try {
			FileInputStream in = new FileInputStream(fn);
			d = new BufferedReader(new InputStreamReader(in));
		}catch(Exception ex) {
			System.out.println("Error while reading file "+fn+ex.toString());
			throw new java.rmi.ServerRuntimeException("Error while reading file "+fn,ex);
		}

		if (d==null) return null;

		try {
		Vector vt = new Vector();

		while (true)
		{
			String line = d.readLine();
			if (line == null) break;
			if (!line.startsWith("Mem")) continue;
				
			StringTokenizer st = new StringTokenizer(line);
			int count = st.countTokens();
			if (count!=7) continue;
			
			st.nextToken();
			long total = Long.parseLong(st.nextToken())/1000;
			long used = Long.parseLong(st.nextToken())/1000;
			long free = Long.parseLong(st.nextToken())/1000;
			long shared = Long.parseLong(st.nextToken())/1000;
			long buffers = Long.parseLong(st.nextToken())/1000;
			long cached = Long.parseLong(st.nextToken())/1000;
			
			vt.addElement(Util.getHostName());
			vt.addElement(new Long(total));
			vt.addElement(new Long(used));
			vt.addElement(new Long(free));
			vt.addElement(new Long(shared));
			vt.addElement(new Long(buffers));
			vt.addElement(new Long(cached));
			break;
		}

		d.close();	
                return vt;
		}catch(Exception ee)
		{
			System.out.println("Retrieve Memory Information error:"+ee.toString());
			addErrorEvent("Retrieve Memory information error:"+ee.toString());
			try {
			if (d!=null) d.close();
			}catch(Exception exp){};
			return null;
		}
        }

	public String[] getEntryNames() {
		return entryNames;
	}

	public String getKeyName() {
		return entryNames[0];
	}

	public int[] getEntryType() {
		return entryTypes;
	}

	public int getTableType() {
		return ResourceTable.SIGLETABLE;
	}

	public Vector getRange(String entryName)
	{
		return null;
	}

	public int getEntryMaxLen(int id)
	{
		return entryMaxLen[id];
	}
	
	public static void main(String[] args) {
		Cluster_Workstation_MemState rs = new Cluster_Workstation_MemState();
		Vector vt = null;

		try {
		vt = rs.getVector();
		}catch(Exception exp){
			System.out.println(exp);
		}

		if (vt!=null)
			System.out.println(vt);
		else
			System.out.println("");

		System.out.println("----------------");
		System.out.println(rs.errorToString());
		System.exit(0);
	}

	public String getDesc()
	{
		return "Memory Stat";
	}

}
