/*
 * @(#) Cluster_Workstation_Network_Stat_IP.java 98/4/23
 *
 * Copyright 1997, 1998 by SRG, Dept. of Computer Science, Hong Kong Univ.
 * Pokfulam RD, Hong Kong, China
 * All rights reserved.
 * 
 * This software is the condifential and prprietary information of SRG. 
 * You shall not disclose such confidential information and shall use it
 * only in accordance with the terms of the license agreement you got this
 * software.
 */

package CLUSTERPROBE.resources;
 
import java.util.*;
import java.io.DataInputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.FileInputStream;

/**
 * Define the resource format of Cluster-Workstation network statistics, include
 * raw, ip, tcp, udp.
 *
 * @author Jimmy Liang
 * @version 1.00 12/23/98
 */

public class Cluster_Workstation_Network_Stat_IP extends ResourceTable
{
	String[] entryNames = {"Host Name","Forwarding","DefaultTTL","InReceives","InHdrErrors","InAddrErrors","ForwDatagrams","InUnknownProtos","InDiscards","InDelivers","OutRequests","OutDiuscards","OutNoRoutes","ReasmTimeout","ReasmReqds","ReasmOKs","ReasmFails","FragOKs","FragFails","FragCreates"};
	int[] entryTypes = {Resource.STRING,Resource.LONG,Resource.LONG,Resource.LONG,Resource.LONG,Resource.LONG,Resource.LONG,Resource.LONG,Resource.LONG,Resource.LONG,Resource.LONG,Resource.LONG,Resource.LONG,Resource.LONG,Resource.LONG,Resource.LONG,Resource.LONG,Resource.LONG,Resource.LONG,Resource.LONG};
	int[] entryMaxLen = {15,5,6,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10};
		
	long[]  prev = new long[19];
	static int UPDATETIME=20*1000;
	static String fn = "/proc/net/snmp";

	static boolean first = false;

	public Cluster_Workstation_Network_Stat_IP() {
		setUpdateTime(UPDATETIME);
		setMinTime(10*1000);
	}

	public Vector getVector() throws java.rmi.ServerRuntimeException
	{
		BufferedReader d = null;

		try {
			FileInputStream in = new FileInputStream(fn);
			d = new BufferedReader(new InputStreamReader(in));
		}catch(Exception ex) {
			System.out.println("Error while reading file "+fn+ex.toString());
			throw new java.rmi.ServerRuntimeException("Error while reading file "+fn,ex);
		}

		if (d==null) return null;

		try {
		Vector vt = new Vector();
		boolean flag = false;

		while (true)
		{
			String line = d.readLine();
			if (line == null) break;
			if (!line.startsWith("Ip:")) continue;

			if (!flag)
			{
				flag = true;
				continue;
			}
				
			line = line.substring(3,line.length());
			StringTokenizer st = new StringTokenizer(line);
			int count = st.countTokens();
			if (count!=19) continue;
			
			vt.addElement(Util.getHostName());

			int mark = 0;
			while (st.hasMoreTokens())
			{
				long num = Long.parseLong(st.nextToken());
				long rnum = num - prev[mark];
				prev[mark++]= num;
				if (rnum<0) rnum = num;

				if (!first)
					rnum = 0;

				vt.addElement(new Long(rnum));
			}
			
			if (!first) first = true;

			break;
		}

		d.close();	
                return vt;
		}catch(Exception ee)
		{
			System.out.println("Retrieve IP Statistics Information error:"+ee.toString());
			addErrorEvent("Retrieve IP Statistics information error:"+ee.toString());
			try {
			if (d!=null) d.close();
			}catch(Exception exp){};
			return null;
		}
        }

	public String[] getEntryNames() {
		return entryNames;
	}

	public String getKeyName() {
		return entryNames[0];
	}

	public int[] getEntryType() {
		return entryTypes;
	}

	public int getTableType() {
		return ResourceTable.SIGLETABLE;
	}

	public Vector getRange(String entryName)
	{
		return null;
	}

	public int getEntryMaxLen(int id)
	{
		return entryMaxLen[id];
	}
	
	public static void main(String[] args) {
		Cluster_Workstation_Network_Stat_IP rs = new Cluster_Workstation_Network_Stat_IP();
		Vector vt = null;

		try {
		vt = rs.getVector();
		}catch(Exception exp){
			System.out.println(exp);
		}

		if (vt!=null)
			System.out.println(vt);
		else
			System.out.println("");

		System.out.println("----------------");
		System.out.println(rs.errorToString());
		System.exit(0);
	}

	public String getDesc()
	{
		return "IP Packets Statistics";
	}

}
