/*
 * @(#) Cluster_Workstation_NicInfo.java 99/4/9
 *
 * Copyright 1997, 1998 by SRG, Dept. of Computer Science, Hong Kong Univ.
 * Pokfulam RD, Hong Kong, China
 * All rights reserved.
 * 
 * This software is the condifential and prprietary information of SRG. 
 * You shall not disclose such confidential information and shall use it
 * only in accordance with the terms of the license agreement you got this
 * software.
 */

package CLUSTERPROBE.resources;
 
import java.util.*;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.DataInputStream;

/**
 * Define the format of Cluste_Workstation_Network Interface Info
 *
 * @author Jimmy Liang
 * @version 1.00 4/9/99
 */

public class Cluster_Workstation_NicInfo extends ResourceTable
{
	String[] entryNames = {"Interface Name", "Link enacap", "HW addr","Inet addr", "Bcast","Mask","State","MTU","Metric","Interrupt","Base addr"};
	int[] entryTypes = {Resource.STRING,Resource.STRING,Resource.STRING,Resource.STRING,Resource.STRING,Resource.STRING,Resource.STRING,Resource.SHORT,Resource.SHORT,Resource.SHORT,Resource.STRING};
		
	int[] entryMaxLen = {10,15,20,18,18,18,40,6,5,5,8};

	public Cluster_Workstation_NicInfo()
	{
		setUpdateTime(24*60*1000);
		setMinTime(2*60*1000);
	}

	public String[] getEntryNames() {
		return entryNames;
	}

	public String getKeyName() {
		return entryNames[0];
	}

	public boolean isValueFromServer() {
		return false;
	}

	public int[] getEntryType() {
		return entryTypes;
	}

	public int getTableType() {
		return ResourceTable.MULTITABLE;
	}

	public Vector getRange(String entryName)
	{
		return null;
	}

	public Hashtable getHashtable() throws java.rmi.ServerRuntimeException
	{
		Hashtable ht = new Hashtable();

		try {
			Process p = Util.runCommand("/sbin/ifconfig");
			try {
				Thread.sleep(500);
			}catch(Exception e0) {};
			int i=0;
			for (;i<6;i++)
			{
				try {
				p.exitValue();
				}catch(Exception e1) {
					try {
					Thread.sleep(1000);
					}catch(Exception e2){};
					continue;
				}
				break;
			}

			if (i>=6) {
				System.out.println("Time out");	
				addErrorEvent("Time out");
				return null;
			}

			String errMessage = Util.getProcessErrMsg(p);
			if (errMessage!=null) {
				System.out.println("execute error:"+errMessage);
				addErrorEvent("execute error:"+errMessage);
				return null;
			}

			DataInputStream in = new DataInputStream(p.getInputStream());
			BufferedReader d = new BufferedReader(new InputStreamReader(in));

			String line = null;

			while ((line = d.readLine())!=null)
			{
				try {
				if (line.indexOf("Link encap")==-1) continue;

				StringTokenizer st = new StringTokenizer(line);
				int count = st.countTokens();
				if (count<3) continue;

				String Iname = st.nextToken();

				if (!st.nextToken().equals("Link"))
					continue;
				StringTokenizer st2 = new StringTokenizer(st.nextToken(),":");
				if (st2.countTokens()!=2) continue;
				st2.nextToken();
				String encap = st2.nextToken();					
				String hwaddr = "";
				boolean hw = false;

				for (int ii=3;ii<count;ii++)
				{
					String ss = st.nextToken();
					if (ss.indexOf("HWaddr")!=-1)
					{
						hw = true;	
						continue;
					}
					if (hw)
					{
						hwaddr = ss;
						break;
					}
					else
						encap = encap +" "+ss;
				}
					
				line = d.readLine();
				if (line==null) continue;
				String iaddr = "";
				String bcast = "";
				String mask = "";	
				
				int index = line.indexOf("inet addr:");
				if (index!=-1)
				{
					st2 = new StringTokenizer(line.substring(index+10,line.length()));
					iaddr = st2.nextToken();
				}

				index = line.indexOf("Bcast:");
				if (index!=-1)
				{
					st2 = new StringTokenizer(line.substring(index+6,line.length()));
					bcast = st2.nextToken();
				}

				index = line.indexOf("Mask:");
				if (index!=-1)
				{
					st2 = new StringTokenizer(line.substring(index+5,line.length()));
					mask = st2.nextToken();
				}

				line = d.readLine();
				if (line==null) continue;
				st2 = new StringTokenizer(line,":");
				count = st2.countTokens();
				if (count!=3) continue;

				StringTokenizer st3 = new StringTokenizer(st2.nextToken());
				count = st3.countTokens();
				String state = "";
				for (int ii=0;ii<count-1;ii++)
					state = state+" "+st3.nextToken();
					
				st3 = new StringTokenizer(st2.nextToken());
				count = st3.countTokens();
				if (count!=2) continue;
				int mtu = Integer.parseInt(st3.nextToken());

				int metric = Integer.parseInt(st2.nextToken());

				line = d.readLine();
				if (line==null) continue;
				line = d.readLine();
				if (line==null) continue;
				
				int interrupt = -1;
				String addr = "";

				line = d.readLine();
				
				if (line!=null)
				{
					st2 = new StringTokenizer(line,":");
					count = st2.countTokens();
					if (count==3)
					{
						st2.nextToken();
						st3 = new StringTokenizer(st2.nextToken());
						interrupt = Integer.parseInt(st3.nextToken());
						addr = st2.nextToken();
					}
				}

				Vector v = new Vector();
				v.addElement(Iname);
				v.addElement(encap);
				v.addElement(hwaddr);
				v.addElement(iaddr);
				v.addElement(bcast);
				v.addElement(mask);				
				v.addElement(state);
				v.addElement(new Integer(mtu));
				v.addElement(new Integer(metric));
				v.addElement(new Integer(interrupt));
				v.addElement(addr);

				ht.put(Iname,v);
				}catch(Exception exp) {
				System.out.println("Error while getting information of NIC"+exp.toString());
				addErrorEvent("Error while getting information of NIC:"+exp.toString());
				}
			}	

			in.close();
			d.close();
			
		}catch(Exception e) {
			System.out.println("Error while getting information of NIC:"+e.toString());
			addErrorEvent("Error while getting information of NIC:"+e.toString());
			return null;
		}
		return ht;
	}

	public int getEntryMaxLen(int id)
	{
		return entryMaxLen[id];
	}

	public static void main(String[] args) {
		Cluster_Workstation_NicInfo rs = new Cluster_Workstation_NicInfo();
		Hashtable ht = null;
		try{
		ht = rs.getHashtable();
		}catch(Exception ee){
			System.out.println(ee);
			System.exit(-1);
		}
		if (ht!=null)
			System.out.println(ht);
		else
			System.out.println("");

		System.out.println("---------");
		System.out.println(rs.errorToString());
		System.exit(0);
	}

	public String getDesc()
	{
		return "Network Configuration";
	}

}
