/*
 * @(#) Cluster_Workstation_RouteTable.java 99/4/9
 *
 * Copyright 1997, 1998 by SRG, Dept. of Computer Science, Hong Kong Univ.
 * Pokfulam RD, Hong Kong, China
 * All rights reserved.
 * 
 * This software is the condifential and prprietary information of SRG. 
 * You shall not disclose such confidential information and shall use it
 * only in accordance with the terms of the license agreement you got this
 * software.
 */

package CLUSTERPROBE.resources;
 
import java.util.*;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.DataInputStream;

/**
 * Define the format of Cluste_Workstation_Network Interface Info
 *
 * @author Jimmy Liang
 * @version 1.00 4/9/99
 */

public class Cluster_Workstation_RouteTable extends ResourceTable
{
	String[] entryNames = {"Destination", "Gateway", "Genmask","Flags", "Metric","Ref","Use","Iface"};
	int[] entryTypes = {Resource.STRING,Resource.STRING,Resource.STRING,Resource.STRING,Resource.SHORT,Resource.SHORT,Resource.SHORT,Resource.STRING};
		
	int[] entryMaxLen = {20,20,20,6,6,6,8,10};

	public Cluster_Workstation_RouteTable()
	{
		setUpdateTime(60*60*1000);
		setMinTime(60*1000);
	}

	public String[] getEntryNames() {
		return entryNames;
	}

	public String getKeyName() {
		return entryNames[0];
	}

	public boolean isValueFromServer() {
		return false;
	}

	public int[] getEntryType() {
		return entryTypes;
	}

	public int getTableType() {
		return ResourceTable.MULTITABLE;
	}

	public Vector getRange(String entryName)
	{
		return null;
	}

	public Hashtable getHashtable() throws java.rmi.ServerRuntimeException
	{
		Hashtable ht = new Hashtable();

		try {
			Process p = Util.runCommand("/sbin/route");
			try {
				Thread.sleep(500);
			}catch(Exception e0) {};
			int i=0;
			for (;i<6;i++)
			{
				try {
				p.exitValue();
				}catch(Exception e1) {
					try {
					Thread.sleep(1000);
					}catch(Exception e2){};
					continue;
				}
				break;
			}

			if (i>=6) {
				System.out.println("Time out");	
				addErrorEvent("Time out");
				return null;
			}

			String errMessage = Util.getProcessErrMsg(p);
			if (errMessage!=null) {
				System.out.println("execute error:"+errMessage);
				addErrorEvent("execute error:"+errMessage);
				return null;
			}

			DataInputStream in = new DataInputStream(p.getInputStream());
			BufferedReader d = new BufferedReader(new InputStreamReader(in));

			String line = null;

			while ((line = d.readLine())!=null)
			{
				try {
				if (line.indexOf("Destination")!=-1) continue;

				StringTokenizer st = new StringTokenizer(line);
				int count = st.countTokens();
				if (count!=8) continue;

				String dest = st.nextToken();
				String gateway = st.nextToken();		
				String mask = st.nextToken();
				String flags = st.nextToken();
				int metric = Integer.parseInt(st.nextToken());
				int ref = Integer.parseInt(st.nextToken());
				int use = Integer.parseInt(st.nextToken());
				String iface = st.nextToken();

				Vector v = new Vector();
				v.addElement(dest);
				v.addElement(gateway);
				v.addElement(mask);
				v.addElement(flags);
				v.addElement(new Integer(metric));
				v.addElement(new Integer(ref));
				v.addElement(new Integer(use));
				v.addElement(iface);

				ht.put(dest,v);
				}catch(Exception exp) {
				System.out.println("Error while getting information of NIC"+exp.toString());
				addErrorEvent("Error while getting information of NIC:"+exp.toString());
				}
			}	

			in.close();
			d.close();
			
		}catch(Exception e) {
			System.out.println("Error while getting information of NIC:"+e.toString());
			addErrorEvent("Error while getting information of NIC:"+e.toString());
			return null;
		}
		return ht;
	}

	public int getEntryMaxLen(int id)
	{
		return entryMaxLen[id];
	}

	public static void main(String[] args) {
		Cluster_Workstation_RouteTable rs = new Cluster_Workstation_RouteTable();
		Hashtable ht = null;
		try{
		ht = rs.getHashtable();
		}catch(Exception ee){
			System.out.println(ee);
			System.exit(-1);
		}
		if (ht!=null)
			System.out.println(ht);
		else
			System.out.println("");

		System.out.println("---------");
		System.out.println(rs.errorToString());
		System.exit(0);
	}

	public String getDesc()
	{
		return "Route Table";
	}
}
