/*
 * @(#) Cluster_Workstation_StatusTable.java 98/11/23
 */

package CLUSTERPROBE.resources;
 
import java.util.*;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;

/**
 * Define the resource format of Cluster-Workstation-StatusTable 
 * @author Jimmy Liang
 * @version 1.00 12/23/98
 */

public class Cluster_Workstation_StatusTable extends ResourceTable
{
	String[] entryNames = {"WorkstationName","State","Description","Load1","Load2","Load3"};
	int[] entryTypes = {Resource.STRING,Resource.SHORT,Resource.STRING,Resource.FLOAT,Resource.FLOAT,Resource.FLOAT};
	int[] entryMaxLen = {16,8,25,6,6,6};
		
	public final static int READY=1;
	public final static int WARNING=2;
	public final static int SWAP_CLOSING_FULL=3;
	public final static int OVERLOAD=4;
	public final static int PACKETLOSING=5;
	public final static int ERROR=100;
	public final static int MALFORMEDURL=101;
	public final static int UNKNOWNHOST=102;
	public final static int CONNECTFAIL=103;
	public final static int RMINOTREADY=104;

	static String[] desc = {"Ready","Ready","Overload","Ready", "Swap_Closing_Full","Ready","Unreach","Ready", "Ready","Unreach","Ready","Ready", "Ready","Package_Losing","Ready","Ready"};

        static int[] st = {1,1,3,1, 3,1,2,1, 1,2,1,1, 1,3,1,1};
	
	static int count = 0;
	static int loop = 0;
	static int tempCount = 0;

	static int UPDATETIME=10000;
	static float loadLimited = (float)1.0;
	static int swapLimited = 500; 
	static int packetsReceived = 0;
	static int inDiscarded=0;
	static int packetsSent = 0;
	static int outDroped=0;
	static double losingLimited = 0.20;
	static boolean firstTime = true;

	Float[] loads = new Float[3];
	Integer state = new Integer(READY);
	String description = "Ready";

	public Cluster_Workstation_StatusTable() {
		setUpdateTime(UPDATETIME);
		setMinTime(5000);
	}

	public Vector getVector() throws java.rmi.ServerRuntimeException
	{
		refresh();

                Vector v = new Vector();
                v.addElement(Util.getHostName());

		v.addElement(state);
		v.addElement(description);
		
                v.addElement(loads[0]);
                v.addElement(loads[1]);
                v.addElement(loads[2]);
		count++;
                return v;
        }

	public void refresh()
	{
		if (firstTime) firstTime = false;

		state = new Integer(READY);
		description = "Ready";

		//refresh loads information, and check if over the load limits.
		getLoads();

		//check if lost packages
		checkPackage();

		//check if swapdisk closing full
		checkSwap();


	}

	public void checkSwap() 
	{
		try {
		Process p = Util.runCommand("/usr/bin/free -k"); 
		String errMessage = Util.getProcessErrMsg(p);
		if (errMessage!=null) {
			System.out.println("process run error:"+errMessage);
			return;
		}

		DataInputStream in = new DataInputStream(p.getInputStream());	
		BufferedReader d = new BufferedReader(new InputStreamReader(in));

		String line = null;
		while ((line = d.readLine())!=null)
			if (line.indexOf("Swap")!=-1) break;
		if (line.indexOf("Swap")==-1) return;
		
		StringTokenizer st = new StringTokenizer(line,":");
		String swapInfo = null;
		while (st.hasMoreTokens())
			swapInfo = st.nextToken();

		StringTokenizer st1 = new StringTokenizer(swapInfo," ");
		if (st1.countTokens()!=3) return;
		st1.nextToken();
		st1.nextToken();
		int freeSwap = Integer.parseInt(st1.nextToken().trim());
		if (freeSwap<=swapLimited) 
		{
			state = new Integer(SWAP_CLOSING_FULL);
			description = "Swap_Closing_Full";
		} 
		p.destroy();
		} catch(Exception e) {
			System.out.println("caused error while call system process:"+e.toString());
		}
	};

	public void checkPackage() 
	{
		
		try {
		int cc  = 0;
		FileInputStream in = new FileInputStream("/proc/net/snmp");
		BufferedReader d = new BufferedReader(new InputStreamReader(in));

		if (d==null) return;

		String line = null;
		boolean flag = false;
		while (true)
		{
			line = d.readLine();
			if (line==null) break;
			if (!line.startsWith("Ip:")) continue;
			if (!flag)
			{
				flag = true;
				continue;
			}

			line = line.substring(3,line.length());
			StringTokenizer st = new StringTokenizer(line);
			if (st.countTokens()!=19) continue;
			
			st.nextToken();
			st.nextToken();
			cc = Integer.parseInt(st.nextToken());
			double receivedInPastSlot = (double)(cc - packetsReceived);
			packetsReceived = cc;

			st.nextToken();
			st.nextToken();
			st.nextToken();
			st.nextToken();
			cc = Integer.parseInt(st.nextToken());
			double discardedInPastSlot = (double) (cc - inDiscarded);
			inDiscarded = cc;

			st.nextToken();
			cc = Integer.parseInt(st.nextToken());
			double sentInPastSlot = (double)(cc - packetsSent);
			packetsSent = cc;

			cc = Integer.parseInt(st.nextToken());
			double dropedInPastSlot = (double)(cc - outDroped);
			outDroped = cc;
	
			if (!firstTime) 
				if ((discardedInPastSlot/receivedInPastSlot)>=losingLimited||(dropedInPastSlot/sentInPastSlot)>=losingLimited)
				{
					state = new Integer(PACKETLOSING);
					description = "packets losing";
				}
			break;
		}
		d.close();
		} catch(Exception e) {
			System.out.println("caused error while call system process:"+e.toString());
		}
	}

	public void getLoads() 
	{
		if (loads[0]==null)
		{
			loads[0] = new Float(0.0);
			loads[1] = new Float(0.0);
			loads[2] = new Float(0.0);
		}

		try {
		Process p = Util.runCommand("/usr/bin/uptime");
		String errMessage = Util.getProcessErrMsg(p);
		if (errMessage!=null) {
			System.out.println("process run error:"+errMessage);
			return;
		}

		DataInputStream in = new DataInputStream(p.getInputStream());	
		BufferedReader d = new BufferedReader(new InputStreamReader(in));
		String line = d.readLine();
		StringTokenizer st = new StringTokenizer(line,":");
		String loadInfo = null;
		while (st.hasMoreTokens())
			loadInfo = st.nextToken();

		StringTokenizer st1 = new StringTokenizer(loadInfo,",");
		if (st1.countTokens()!=3) return;
		for (int i=0;i<3;i++) 
			loads[i] = Float.valueOf(st1.nextToken().trim());
		if (loads[0].floatValue()>=loadLimited)
		{
			state = new Integer(OVERLOAD);	
			description = "Overload";
		}
		
		p.destroy();
		} catch(Exception e) {
			System.out.println("caused error while call system process:"+e.toString());
		}
	}

	public String[] getEntryNames() {
		return entryNames;
	}

	public String getKeyName() {
		return entryNames[0];
	}

	public int[] getEntryType() {
		return entryTypes;
	}

	public int getTableType() {
		return ResourceTable.SIGLETABLE;
	}

	public Vector getRange(String entryName)
	{
		return null;
	}

	public Vector getVectorExceptionProcess(String host, Exception e)
	{
		Vector v = new Vector();
		
		v.addElement(host);

		if (e instanceof java.net.MalformedURLException)
		{
			v.addElement(new Integer(MALFORMEDURL));
			v.addElement("Malformed URL");
		}

		if (e instanceof java.rmi.UnknownHostException)
		{
			v.addElement(new Integer(UNKNOWNHOST));
			v.addElement("Unkown host");
		}

		if (e instanceof java.rmi.ConnectIOException)
		{
System.out.println(e.toString());
			v.addElement(new Integer(CONNECTFAIL));
			v.addElement("Unreachable");
		}
		else {
			v.addElement(new Integer(RMINOTREADY));
			v.addElement("RMI not ready");
		}
		v.addElement(new Float(10.0));
		v.addElement(new Float(10.0));
		v.addElement(new Float(10.0));
		return v;
	}

	public int getEntryMaxLen(int id)
	{
		return entryMaxLen[id];
	}
	
	public static void main(String[] args) {
		Cluster_Workstation_StatusTable rs = new Cluster_Workstation_StatusTable();
		rs.refresh();
System.out.println(rs.state+": "+rs.description);
try {
Thread.sleep(5000);
}catch(Exception e){};
		rs.refresh();
System.out.println(rs.state+": "+rs.description);
	}

	public String getDesc()
	{
		return "Load Status";
	}
}
