/*
 * @(#) Resource.java 99/1/5
 */

package CLUSTERPROBE.resources;
 
import java.util.*;
import java.io.Serializable;
import java.io.IOException;
import CLUSTERPROBE.message.*;

/**
 * Define the format of cluster resources.
 */

public abstract class Resource extends Thread implements Serializable
{
	public final static int SHORT = 0; 
	public final static int LONG = 1;
	public final static int STRING = 2;
	public final static int BOOLEAN = 3;
	public final static int FLOAT = 4;
	public final static int DOUBLE = 5;


	public final static int NODE=1;
	public final static int GROUP=2;
	public final static int JOB=3;
	public final static int QUEUE=4;
	public final static int STATUS=5;

	public final static int SYSTEMLOG=20;
	public final static int DEVICEINFO=21;
	public final static int DISKINFO=22;
	public final static int EXPORT=23;
	public final static int FSTABLE=24;
	public final static int IOPORT=25;
	public final static int KERNELSYMBOL=26;
	public final static int MODULE=27;
	public final static int NICINFO=28;
	public final static int USER=29;
	public final static int VERSION=30;

	public final static int CPUINFO=40;
	public final static int CPUSTATE=41;
	public final static int CPUTEMP=42;
	public final static int FANSPEED=43;

	public final static int LOGININFO=60;
	public final static int MEMSTATE=61;
	public final static int PROCESSINFO=62;

	public final static int ARP=80;
	public final static int ROUTE=81;
	public final static int IPTABLE=82;
	public final static int RAWTABLE=83;
	public final static int TCPTABLE=84;
	public final static int UDPTABLE=85;
	public final static int UNIXTABLE=86;
	public final static int IPSTAT=90;
	public final static int ICMPSTAT=91;
	public final static int TCPSTAT=92;
	public final static int UDPSTAT=93;

	boolean tableFormat = true;
	Date refreshedTime = null;

	Vector errorVector = null;

	int updateTime = -1;

	int minTime = 10;

	Vector range = null;
	
	public void setRefreshedTime(Date d) {
		refreshedTime = d;
	}

	public Date getRefreshedTime() {
		return refreshedTime;
	}

	public void setUpdateTime(int t) {
		updateTime = t;
	}

	public int getUpdateTime() {
		return updateTime;
	}

	public void setRange(Vector r) {
		range = r;
	}

	static public String[] nodeLists = {"c414a1","c414b1"}; 
	static public Vector qservers = new Vector();
	static public Hashtable groupToHost = new Hashtable();
	public Vector getRange() {
		if (range==null) {
			range = new Vector();
			for (int i=0;i<nodeLists.length;i++)
			range.addElement(nodeLists[i]);
		}
		return range;
	}

	// Used to get resource value
	public Object getObject() throws java.rmi.ServerRuntimeException {return null;}
	public Object getObjectExceptionProcess(Exception e) { return null;}
	public Vector getVector() throws java.rmi.ServerRuntimeException {return null;}
	public Vector getVectorExceptionProcess(String host, Exception e) { return null;}
	public Hashtable getHashtable() throws java.rmi.ServerRuntimeException {return null;}
	public Hashtable getHashtableExceptionProcess(String host, Exception e) {return null;}

	public abstract Vector packRefreshedValue();
 
	public abstract String getDesc();

	public void run() {
		System.out.println("In Resource Thread:"+getResourceName()+"...");
		while (true) {

			if (getUpdateTime()<0) 
			{
				System.out.println("getUpdateTime<0:out of ResourceThread:"+getResourceName()+".");
				break;
			}
			try {
				Thread.sleep(getUpdateTime());
			}catch(Exception e) {};

			System.out.println(getResourceName()+" is refreshing now!");
			Vector v = packRefreshedValue();

			// Check if local handler observe the resource
			Vector handlers = Util.getHandlers(getResourceName());
			if (handlers!=null&&v!=null)
			{
				Enumeration evts = v.elements();
				while (evts.hasMoreElements())
				{
					JessicEvent eo = (JessicEvent) evts.nextElement();			
					Enumeration hds = handlers.elements();
					while (hds.hasMoreElements())
					{
						EventHandler handler = (EventHandler)hds.nextElement();
						handler.handleEvent(eo);
					}	
				}
			}

			// Check the client which interested with this event and Send event to them				
			if (v!=null)
			{
				try {
				Enumeration en = Util.registryTable.keys();
				while (en.hasMoreElements())
				{
					EventTransceiver transceiver = (EventTransceiver) en.nextElement();
					Vector vt = (Vector) Util.registryTable.get(transceiver);
					Enumeration registers = vt.elements();
					while (registers.hasMoreElements())
					{
						ResourceRegistryEvent reg = (ResourceRegistryEvent) registers.nextElement();
						if (getResourceName().equals(reg.getName())) 
						{
							try {
							transceiver.sendEvents(v);
							break;
							}catch(IOException ex) {
								// if can't touch client, then remove the transceiver and corresponding registers
//System.out.println("remove transceiver from table");
								Util.registryTable.remove(transceiver);
								Util.removeRegister(reg.getName());
							}
						}
					}	
				}
				}catch(Exception ex) {
					System.out.println("Send events error:"+ex.toString());
				}
			}

			setRefreshedTime(Calendar.getInstance().getTime());
		}
		System.out.println("End of Resource Thread of :"+getResourceName()+".");
	}	
			
	public String getResourceName() 
	{
		Class cl = this.getClass();
		return cl.getName().trim();
	}

	public boolean isTable() {
		return tableFormat;
	}

	public void addErrorEvent(String em)
	{
		if (errorVector == null) errorVector = new Vector();
		errorVector.addElement(new ErrorEvent(getResourceName(),em));
	}

	public void setMinTime(int t) {
		minTime = t;
	}

	public int getMinTime()
	{
		return minTime;
	}
	
	public boolean isValueFromServer() {
		return false;
	}

	public static String getNameFromID(short id)
	{
		switch (id)
		{
		case NODE:
			return "CLUSTERPROBE.resources.Cluster_NodeLists";
		case GROUP:
			return "CLUSTERPROBE.resources.Cluster_GroupLists";
		case JOB:
			return "CLUSTERPROBE.resources.Cluster_JQS_JobStat";
		case QUEUE:
			return "CLUSTERPROBE.resources.Cluster_JQS_QueueStat";
		case STATUS:
			return "CLUSTERPROBE.resources.Cluster_Workstation_StatusTable";
		case SYSTEMLOG:
			return "CLUSTERPROBE.resources.Cluster_SystemLog";
		case ARP:
			return "CLUSTERPROBE.resources.Cluster_Workstation_ArpTable";
		case CPUINFO:
			return "CLUSTERPROBE.resources.Cluster_Workstation_CPUInfo";
		case CPUTEMP:
			return "CLUSTERPROBE.resources.Cluster_Workstation_CPUTemp";
		case CPUSTATE:
			return "CLUSTERPROBE.resources.Cluster_Workstation_CPUState";
		case DEVICEINFO:
			return "CLUSTERPROBE.resources.Cluster_Workstation_DeviceInfo";
		case DISKINFO:
			return "CLUSTERPROBE.resources.Cluster_Workstation_DiskInfo";
		case EXPORT:
			return "CLUSTERPROBE.resources.Cluster_Workstation_Expoprts";
		case FANSPEED:
			return "CLUSTERPROBE.resources.Cluster_Workstation_FanSpeed";
		case FSTABLE:
			return "CLUSTERPROBE.resources.Cluster_Workstation_FsTable";
		case IOPORT:
			return "CLUSTERPROBE.resources.Cluster_Workstation_IOPorts";
		case KERNELSYMBOL:
			return "CLUSTERPROBE.resources.Cluster_Workstation_KernelSymbols";
		case LOGININFO:
			return "CLUSTERPROBE.resources.Cluster_Workstation_LoginInfo";
		case MEMSTATE:
			return "CLUSTERPROBE.resources.Cluster_Workstation_MemState";
		case MODULE:
			return "CLUSTERPROBE.resources.Cluster_Workstation_Modules";
		case IPTABLE:
			return "CLUSTERPROBE.resources.Cluster_Workstation_Network_ContTable_IP";
		case UDPTABLE:
			return "CLUSTERPROBE.resources.Cluster_Workstation_Network_ContTable_UdP";
		case TCPTABLE:
			return "CLUSTERPROBE.resources.Cluster_Workstation_Network_ContTable_Tcp";
		case RAWTABLE:
			return "CLUSTERPROBE.resources.Cluster_Workstation_Network_ContTable_Raw";
		case UNIXTABLE:
			return "CLUSTERPROBE.resources.Cluster_Workstation_Network_ContTable_Unix";
		case IPSTAT:
			return "CLUSTERPROBE.resources.Cluster_Workstation_Network_Stat_IP";
		case ICMPSTAT:
			return "CLUSTERPROBE.resources.Cluster_Workstation_Network_Stat_Icmp";
		case TCPSTAT:
			return "CLUSTERPROBE.resources.Cluster_Workstation_Network_Stat_Tcp";
		case UDPSTAT:
			return "CLUSTERPROBE.resources.Cluster_Workstation_Network_Stat_Udp";
		case NICINFO:
			return "CLUSTERPROBE.resources.Cluster_Workstation_NicInfo";
		case PROCESSINFO:
			return "CLUSTERPROBE.resources.Cluster_Workstation_ProcessInfo";
		case ROUTE:
			return "CLUSTERPROBE.resources.Cluster_Workstation_RouteTable";
		case USER:
			return "CLUSTERPROBE.resources.Cluster_Workstation_UserInfo";
		case VERSION:
			return "CLUSTERPROBE.resources.Cluster_Workstation_Version";
	}
	return null;
}

	public static short getIDFromName(String name)
	{
		if (name==null) return 0;

		if (name.equals("CLUSTERPROBE.resources.Cluster_NodeLists"))
			return  NODE;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_GroupLists"))
			return GROUP;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_JQS_JobStat"))
			return JOB;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_JQS_QueueStat"))
			return QUEUE;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_StatusTable"))
			return STATUS;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_SystemLog"))
			return SYSTEMLOG;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_ArpTable"))
			return ARP;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_CPUInfo"))
			return CPUINFO;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_CPUTemp"))
			return CPUTEMP;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_CPUState"))
			return CPUSTATE;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_DeviceInfo"))
			return DEVICEINFO;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_DiskInfo"))
			return  DISKINFO;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_Expoprts"))
			return EXPORT;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_FanSpeed"))
			return FANSPEED;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_FsTable"))
			return FSTABLE;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_IOPorts"))
			return IOPORT;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_KernelSymbols"))
			return KERNELSYMBOL;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_LoginInfo"))
			return  LOGININFO;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_MemState"))
			return MEMSTATE;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_Modules"))
			return  MODULE;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_Network_ContTable_IP"))
			return IPTABLE;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_Network_ContTable_UdP"))
			return UDPTABLE;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_Network_ContTable_Tcp"))
			return TCPTABLE;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_Network_ContTable_Raw"))
			return RAWTABLE;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_Network_ContTable_Unix"))
			return UNIXTABLE;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_Network_Stat_IP"))
			return  IPSTAT;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_Network_Stat_Icmp"))
			return ICMPSTAT;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_Network_Stat_Tcp"))
			return TCPSTAT;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_Network_Stat_Udp"))
			return UDPSTAT;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_NicInfo"))
			return NICINFO;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_ProcessInfo"))
			return PROCESSINFO;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_RouteTable"))
			return ROUTE;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_UserInfo"))
			return USER;
		else if (name.equals("CLUSTERPROBE.resources.Cluster_Workstation_Version"))
			return VERSION;
		return 0;
	}


}



	
	
