/*
 * @(#) ResourceTable.java 98/11/23
 */

package CLUSTERPROBE.resources;
 
import java.util.*;
import java.io.Serializable;

import CLUSTERPROBE.message.ResourceTableData; 
import CLUSTERPROBE.message.ResourceRefreshEvent;
import CLUSTERPROBE.message.ErrorEvent;

/**
 * Define the interface for the resource showed by table , include getting 
 * resource value , packing and unpacking data package. 
 */

public abstract class  ResourceTable extends Resource implements Serializable
{
	public static final int MULTITABLE = 1;
	public static final int SIGLETABLE = 2;
	public static final int timeout = 10000;
	private boolean DebugFlag = false;

	// Get all entry names
	public abstract String[] getEntryNames();

	// Get table type
	public abstract int getTableType();

	// Get all entry type
	public abstract int[] getEntryType();

	// Get the range of the specified entry's value 
	public abstract Vector getRange(String entryName);

	// Get key name
	public abstract String getKeyName();
	
	public Hashtable getTable() {return null;};

	public String[] getTarget() {
		Vector v = getRange();
		String[] target = new String[v.size()];
		Enumeration e = v.elements();
		int i=0;
		while (e.hasMoreElements())
			target[i++]=(String)e.nextElement(); 
		return target;
	}

	// Get the current table values 
	public Hashtable getResourceValue()
	{
		Hashtable ht = new Hashtable();
		String rn = this.getClass().getName();
	
		if (getTableType()==SIGLETABLE)
		{
			if (isValueFromServer())
				ht = getTable();
			else
			{
				String[] target = getTarget();
				Vector threads = new Vector();
				Vector hts = new Vector();
				for (int i=0;i<target.length;i++)
				{
					threads.addElement(Util.getResourceRecord(target[i],this));
					hts.addElement(target[i]);
				}

				int count = 0;
				while (true)
				{
					if (threads.size()==0)
						break;
					try{
					Thread.sleep(100);
					}catch(Exception ee){};
					count+=100;

					Enumeration en = threads.elements();
					Enumeration en1 = hts.elements();
					while (en.hasMoreElements())
					{
						try {
						RMIThread t = (RMIThread) en.nextElement();
						String hn = (String) en1.nextElement();
						if (!t.isAlive())
						{
							Vector v = t.getVector();
							if (v!=null)
								ht.put(getValue(getId(getKeyName()),v),v);
							threads.removeElement(t);
							hts.removeElement(hn);
						}
						}catch(Exception e1) {};
					}
					if (count>timeout) break;
				}
				for (int kk=0;kk<hts.size();kk++)
				{
					String hn = (String) hts.elementAt(kk);
					Vector v = getVectorExceptionProcess(hn,new java.rmi.ConnectIOException("Time out"));	
					if (v!=null)
						ht.put(getValue(getId(getKeyName()),v),v);
				}
			}
		}
		else 
		{
			if (isValueFromServer())
				ht = getTable();
			else 
			{
				String[] target = getTarget();
				Vector threads = new Vector();
				Vector hts = new Vector();
				for (int i=0;i<target.length;i++)
				{
					threads.addElement(Util.getResourceTable(target[i],this));
					hts.addElement(target[i]);
				}
				
				int count = 0;
				while (true)
				{
					if (threads.size()==0) break;
					try{
					Thread.sleep(100);
					}catch(Exception ee){};
					count+=100;

					Enumeration en = threads.elements();
					Enumeration en1 = hts.elements();
					while (en.hasMoreElements())
					{
						try{
						RMIThread t = (RMIThread) en.nextElement();
						String hn = (String) en1.nextElement();
						if (!t.isAlive())
						{
							Hashtable hb = t.getHashtable();
							if (hb!=null)
								ht.put(hn,hb);
							threads.removeElement(t);
							hts.removeElement(hn);
						}
						}catch(Exception e1){};
					}
					if (count>timeout) break;
				}
				for (int kk=0;kk<hts.size();kk++)
				{ 
					String hn = (String) hts.elementAt(kk);
					System.out.println("Time out"+hn);
					Hashtable hb = getHashtableExceptionProcess(hn,new java.rmi.ConnectIOException("Time out"));
					if (hb!=null)
						ht.put(hn,hb);
				}
			}
		}	
		return ht;
	}


	// pack the different parts from the current values to old values
	public synchronized Vector packRefreshedValue() {
		errorVector = null;
		Hashtable nowValue = getResourceValue();
		String rn = getResourceName();
		Enumeration nv = nowValue.keys();
		if(DebugFlag) System.out.println("1");
		Vector v = new Vector();
		long time = Calendar.getInstance().getTime().getTime();

		if (errorVector != null) {
			Enumeration errs = errorVector.elements();
			while (errs.hasMoreElements())
				v.addElement(errs.nextElement());
		}
		if(DebugFlag) System.out.println("2");

		if (getTableType() ==  SIGLETABLE) 
		{
			ResourceRefreshEvent evt = new ResourceRefreshEvent("",time);
			while (nv.hasMoreElements())
			{
				Object k = nv.nextElement();
				Vector record = (Vector)nowValue.get(k);
				evt.addResource(new ResourceTableData(rn,k,record));
			}
			v.addElement(evt);
		if(DebugFlag) System.out.println("3");

			return v;
		}

		else {
			while (nv.hasMoreElements()) 
			{
				Object k = nv.nextElement();
				ResourceRefreshEvent evt = new ResourceRefreshEvent(k,time);
				Hashtable ht=(Hashtable)nowValue.get(k);
				Enumeration ht_key = ht.keys();
				while (ht_key.hasMoreElements())
				{
					Object kk = ht_key.nextElement();
					evt.addResource(new ResourceTableData(rn,kk,(Vector)ht.get(kk)));
				}
				if (!nv.hasMoreElements())
					evt.setFinalOne(true);
				else
					evt.setFinalOne(false);
				v.addElement(evt);
			}
			if(DebugFlag) System.out.println("4");
			return v;
		}
	}
					
	public boolean isSameRecord(Vector n, Vector o)
	{
		try {
			Enumeration newList = n.elements();
			Enumeration oldList = o.elements();
			while (newList.hasMoreElements())
			{
				Object newObject = newList.nextElement();
				Object oldObject = oldList.nextElement();
				if (newObject!=null&&oldObject!=null) 
				{
					if (!newObject.equals(oldObject))
						return false;	
				}
				if (newObject!=null&&oldObject==null) 
					return false;
			}
			return true;
		}catch(Exception e) {};
		return false;
	}
		
	public Object getValue(int id,Vector vt)
	{
		try {
		return vt.elementAt(id);
		}catch(Exception e){};
		return null;
	}

	public int getType(int id)
	{
		return getEntryType()[id];
	}


	public int getId(String en) {
		String[] entrys = getEntryNames();
		for (int i=0;i<entrys.length;i++)
			if (en.trim().toLowerCase().equals(entrys[i].trim().toLowerCase()))
				return i;
		return -1;
	}

	public String entryToString()
	{
		String[] eNames = getEntryNames();
		StringBuffer strb = new StringBuffer();

		for (int i=0;i<eNames.length;i++)
		{
			strb.append(eNames[i]);
			if (eNames[i].length()<getEntryMaxLen(i)) 
				for (int ii=0;ii<(getEntryMaxLen(i)-eNames[i].length());ii++)
					strb.append(" ");
			else 
				strb.append(" ");
		}
		strb.append("\n");
		return new String(strb);
	}

	public String entryToHTMLString()
	{
		String[] eNames = getEntryNames();
		StringBuffer strb = new StringBuffer();

		strb.append("<tr>\n");
		for (int i=0;i<eNames.length;i++)
			strb.append("<td bgcolor=#f0f000 align=center nowrap><b><font color=#0000ff size=+1>"+eNames[i]+"</font></b></td>\n");
		strb.append("</tr>\n\n");
		return new String(strb);
	}

	public String toString(Vector v)
	{
		StringBuffer strb = new StringBuffer();
		String[] eNames = getEntryNames();

		try {
		for (int i=0;i<eNames.length;i++)
		{
			
			try {
			Object ob = v.elementAt(i);
			if (ob!=null) {
				String temp = ob.toString();
				strb.append(ob);
				if (temp.length()<getEntryMaxLen(i)) 
					for (int ii=0;ii<(getEntryMaxLen(i)-temp.length());ii++)
						strb.append(" ");
				else 
					strb.append(" ");
			}
			else {
				for(int ii=0;ii<getEntryMaxLen(i);ii++)
					strb.append(" ");
			}
			}catch(Exception e1) {
				for(int ii=0;ii<getEntryMaxLen(i);ii++)
					strb.append(" ");
			}
				
		}
		strb.append("\n");
		}catch(Exception e){
			System.out.println(e);
			e.printStackTrace();
		}
		return new String(strb);
	}
		
	public String toHTMLString(Vector v)
	{
		StringBuffer strb = new StringBuffer();
		String[] eNames = getEntryNames();

		try {
		strb.append("<tr>\n");
		for (int i=0;i<eNames.length;i++)
		{
			
			try {
			Object ob = v.elementAt(i);
			if (ob!=null) 
			{
				String temp = ob.toString().trim();
				if (temp!=null&&temp.length()!=0)
					strb.append("<td><center>"+ob.toString()+"</center></td>\n");
				else
					strb.append("<td><center>\\</center></td>\n");
			}
			else 
				strb.append("<td><center>\\</center></td>\n");
			}catch(Exception e1) {
					strb.append("<td><center>\\</center></td>\n");
			}
		}
		strb.append("</tr>\n\n");
		}catch(Exception e){
			System.out.println(e);
			e.printStackTrace();
		}
		return new String(strb);
	}
				
	public int getEntryMaxLen(int id)
	{
		return 16;
	}

	public String errorToString()
	{
		if (errorVector==null) return null;
		Enumeration errs = errorVector.elements();
		StringBuffer strb = new StringBuffer();
		try {
			while (errs.hasMoreElements())
			{
				ErrorEvent evt = (ErrorEvent) errs.nextElement();	
				strb.append(evt.getErrMessage()+"\n");	
			}
		}catch(Exception e) {};
		return new String(strb);
	}
}
		
