/*
 * @(#) Util.java 99/1/6
 */

package CLUSTERPROBE.resources;
 
import java.util.*;
import java.io.Serializable;
import java.rmi.*;
import java.rmi.server.*;
import java.io.DataInputStream;
import java.net.InetAddress;

import CLUSTERPROBE.message.EventHandler;
import CLUSTERPROBE.rmi.*;

/**
 * define some utilities such as calling remote method
 *
 * @author Jimmy Liang
 * @version 1.00 1/6/99
 */

public class Util extends Object
{
	// store all running resource objects
	public static Hashtable resourceTable = new Hashtable();

	// store all registered observers
	public static Hashtable registryTable = new Hashtable();

	// store all concerned resources
	public static Hashtable resourceRegisterTable = new Hashtable();

	// store all touched remote objects
	public static Hashtable rmiInterfaces = new Hashtable();

	// store local observers
	public static Hashtable localHandlers = new Hashtable();

	static int GETVECTOR = 0;
	static int GETHASHTABLE = 1;
	static int timeout = 5000;
	
	static boolean DebugFlag = true;

	public static RMIThread getResourceRecord(String host, Resource rs)
	{
	
		// try to call remote method to get vector
		RMIThread t = new RMIThread(host,rs,GETVECTOR);
		//t.start();
		t.RMIDo();
		return t;
	}
	
	public static RMIThread getResourceTable(String host, Resource rs)
	{
		// try to call remote method to get a table 
		RMIThread t = new RMIThread(host,rs,GETHASHTABLE);
		//t.start();
		t.RMIDo();
		return t;
	}
	
	public static boolean contains(Resource r) {
		String rn = r.getResourceName();
		return contains(rn);
	}

	public static boolean contains(String rn) {
		return resourceTable.containsKey(rn);
	}

	public static void addResource(Resource r) {
		if (contains(r))
			resourceTable.remove(r.getResourceName());
		resourceTable.put(r.getResourceName(),r);
		resourceRegisterTable.put(r.getResourceName(),new Integer(1));
	}
		
	public static Resource getResource(String rn) throws Exception {
		if (!contains(rn))
			return (Resource)instanceClass(rn);
		return (Resource) resourceTable.get(rn);
	}

	public static void removeRegister(String rn)
	{
		try {
			if (resourceRegisterTable.containsKey(rn))
			{
				int count = ((Integer)resourceRegisterTable.get(rn)).intValue()-1;
		System.out.println(rn+" count is "+count);
				if (count<=0)
				{
					resourceRegisterTable.remove(rn);
					if (contains(rn))
					{
						Resource rs = (Resource) resourceTable.get(rn);
						resourceTable.remove(rn);
		System.out.println("stop "+rs);
						rs.stop();
					}
				}
				else
					resourceRegisterTable.put(rn,new Integer(count));
			}			
		}catch(Exception e)
		{
			System.out.println("removeRegister:"+e.toString());
		}
	}
					
	public static void addRegister(String rn) {
		if (resourceRegisterTable.containsKey(rn))
		{
			int count = ((Integer)resourceRegisterTable.get(rn)).intValue();
			resourceRegisterTable.put(rn,new Integer(count+1));
		}
		else
			resourceRegisterTable.put(rn,new Integer(1));
	}

	public static Object instanceClass(String cn) 
			throws ClassNotFoundException, InstantiationException, IllegalAccessException, Exception
	{
		Class cl = null;
		try {
		cl = Class.forName(cn);
		//Class cl = RMIClassLoader.loadClass(cn);	
		}catch(ClassNotFoundException e) {
			return getClassFromServer(cn);
		}	
		return cl.newInstance();
	}

	public static ServerRMIInterface server = null;
	
	public static Object getClassFromServer(String cn) 
			throws ClassNotFoundException, InstantiationException, IllegalAccessException, Exception
	{
		if (server==null)
		{
			String des = "rmi://"+RemoteServer.getClientHost()+":3000/server";
System.out.println(des);
			server = (ServerRMIInterface)Naming.lookup(des);
		}
		return server.loadObject(cn);
	}
		
	public static NodeRMIInterface getRMIInterface(String host) throws Exception
	{
			System.out.println("get "+host+" interface!");
			boolean tmp = rmiInterfaces.containsKey(host);
			if(DebugFlag) System.out.println(" rmiInterface contains "+host+" :"+tmp); 
			if(!tmp)
			//if (!rmiInterfaces.containsKey(host))
			{
				String des = "rmi://"+host+":3000/agent";
				rmiInterfaces.put(host,(NodeRMIInterface) Naming.lookup(des));
			}
			else {
				try {
				    ((NodeRMIInterface) rmiInterfaces.get(host)).test();
				}catch(Exception e) {
					String des = "rmi://"+host+":3000/agent";
					removeRMIInterface(host);
					rmiInterfaces.put(host,(NodeRMIInterface) Naming.lookup(des));
				}
			}
			System.out.println("successful got "+host+" interface!");
			return (NodeRMIInterface) rmiInterfaces.get(host);
	}

	public static void removeRMIInterface(String host) 
	{
		if (rmiInterfaces.containsKey(host))
			rmiInterfaces.remove(host);
	}

	public static void main(String[] argv)
	{
	System.setSecurityManager(new MySecurityManager());
	try {
		//Util.instanceClass("CLUSTERPROBE.resources.Cluster_Workstation_StatusTable");
		NodeRMIInterface nd = Util.getRMIInterface("c414b1");
		
	System.out.println("ok!");
	}catch(Exception e) {
		System.out.println(e);
	}
}


public static Process runCommand(String command) throws Exception
{
	return Runtime.getRuntime().exec(command);
}

public static String getProcessErrMsg(Process p) throws Exception
{
	DataInputStream err = new DataInputStream(p.getErrorStream());
	int count = err.available();
	if (count>0) {
		byte[] b = new byte[count+1];
		err.readFully(b);
		return new String(b);
	}
	return null;
}

public static String getProcessMsg(Process p) throws Exception
{
	DataInputStream output = new DataInputStream(p.getInputStream());
	int count = output.available();
	if (count>0)
	{
		byte[] b = new byte[count+1];
		output.readFully(b);
		return new String(b);
	}
	return null;
}

// Get local handlers for specified resource
 public static Vector getHandlers(String rn)
 {
	if (!localHandlers.containsKey(rn)) return null;	
	return (Vector)localHandlers.get(rn);
 }

// if some local observers concern the resource
 public static boolean containsLocalHandler(String rn)
 {
	return localHandlers.containsKey(rn);
 }

// Add new local observers to specified resource
 public static void addLocalHandler(EventHandler h, String rn)
 {
	Vector vt = getHandlers(rn);
	if (vt==null) vt = new Vector();
	if (!vt.contains(h))
		vt.addElement(h);
	localHandlers.put(rn,vt);
 }

// Cancel a observer on a specified resource
 public static void removeLocalHandler(EventHandler h, String rn)
 {
	Vector vt = getHandlers(rn);
	if (vt!=null)
		vt.removeElement(h);
	if (vt.size()==0)
		localHandlers.remove(rn);
 }

// Cancel all observers on a specify resource
public static void removeLocalHandler(String rn)
{
	localHandlers.remove(rn);
}

//Cancel a observer
public static void removeLocalHandler(EventHandler h)
{
	try {
	Enumeration en = localHandlers.keys();
	while (en.hasMoreElements())
	{
		String rn = (String) en.nextElement();
		removeLocalHandler(h,rn);
	}
	}catch(Exception e) {};
}


static String hostName = null;
public static String getHostName() {
	if (hostName != null) return hostName;
	try {
		hostName = InetAddress.getLocalHost().getHostName();
	}catch(Exception ex) {};
	return hostName;
}

}

